#! /usr/bin/env python

import numpy as np
import os
from astropy.io import fits
import matplotlib.pyplot as plt

from matplotlib.colors import LogNorm
import matplotlib.colors as colors

import argparse
import subprocess as sp


from clusex.lib.check import CheckSatReg2
from clusex.lib.check import GetAxis 
from clusex.lib.check import CheckFlag 
from clusex.lib.check import GetWCS 
from clusex.lib.check import GetCounts
from clusex.lib.ds9 import ds9kron




def MakeMask(maskimage, catfile, scale, offset, regfile):
    "Create a mask image using ellipses for every Object of catfile. Now includes offset"
# k Check

    checkflag = 0
    flagsat = 4  # flag value when object is saturated (or close to)

    regflag = 0  # flag for saturaded regions

    n, alpha, delta, xx, yy, mg, kr, fluxrad, ia, ai, e, theta, bkgd, idx, flg, sxmin, sxmax, symin, symax, sxsmin, sxsmax, sysmin, sysmax = np.genfromtxt(
        catfile, delimiter="", unpack=True)

    n = n.astype(int)
    flg = flg.astype(int)

    print("Creating Masks ... \n")

    Rkron = scale * ai * kr + offset

    mask = Rkron < 1
    if mask.any():
        Rkron[mask] = 1

    hdu = fits.open(maskimage)
    img = hdu[0].data

    count1 = count2 = 0

    for idx, val in enumerate(n):

        # check if object doesn't has saturaded regions
        checkflag = CheckFlag(flg[idx], flagsat)
        # check if object is inside of a saturaded box region indicated by
        # user in ds9
        #regflag = CheckSatReg(xx[idx], yy[idx], Rkron[idx], theta[idx], e[idx],regfile)
        regflag = CheckSatReg2(xx[idx], yy[idx],regfile)



        if (checkflag == False) and (regflag == False):

            count1 += 1
            img = MakeKron(img, n[idx], xx[idx], yy[idx], Rkron[idx], theta[idx], e[
                idx], sxsmin[idx], sxsmax[idx], sysmin[idx], sysmax[idx])

        elif(checkflag == True or regflag == True):
            count2 += 1
            

    print ("Number of ellipse masks created: {}  ".format(count1))
    print ("Number of objects rejected because they are saturated: {} \n".format(count2))

    hdu[0].data = img
    hdu.writeto(maskimage, overwrite=True)
    hdu.close()

    return True


def MakeKron(imagemat, idn, x, y, R, theta, ell, xmin, xmax, ymin, ymax):
    "This subroutine create a Kron ellipse within a box defined by: xmin, xmax, ymin, ymax"

# Check

    xmin = np.int(xmin)
    xmax = np.int(xmax)
    ymin = np.int(ymin)
    ymax = np.int(ymax)

    q = (1 - ell)
    bim = q * R

    theta = theta * np.pi / 180  # Rads!!!

    ypos, xpos = np.mgrid[ymin - 1:ymax, xmin - 1:xmax]

    dx = xpos - x
    dy = ypos - y

    landa = np.arctan2(dy, dx)

    mask = landa < 0
    if mask.any():
        landa[mask] = landa[mask] + 2 * np.pi

    landa = landa - theta

    angle = np.arctan2(np.sin(landa) / bim, np.cos(landa) / R)

    xell = x + R * np.cos(angle) * np.cos(theta) - bim * \
        np.sin(angle) * np.sin(theta)
    yell = y + R * np.cos(angle) * np.sin(theta) + bim * \
        np.sin(angle) * np.cos(theta)

    dell = np.sqrt((xell - x)**2 + (yell - y)**2)
    dist = np.sqrt(dx**2 + dy**2)

    mask = dist < dell
    imagemat[ypos[mask], xpos[mask]] = idn

    return imagemat


def MakeSatBox(maskimage, region, val, ncol, nrow):
    "Create a mask for saturated regions"
    "Regions must be in DS9 box regions format"

# k Check

#	fileflag=1

    hdu = fits.open(maskimage)
    img = hdu[0].data

    with open(region) as f_in:

        next(f_in)
#        next(f_in)
#        next(f_in)

        # All lines including the blank ones
        lines = (line.rstrip() for line in f_in)
        lines = (line.split('#', 1)[0] for line in lines)  # remove comments
        # remove lines containing only comments
        lines = (line.rstrip() for line in lines)
        lines = (line for line in lines if line)  # Non-blank lines

        for line in lines:

            (box, info) = line.split('(')

            if(box == "box"):

                (xpos, ypos, xlong, ylong, trash) = info.split(',')

                xpos = float(xpos)
                ypos = float(ypos)
                xlong = float(xlong)
                ylong = float(ylong)

                xlo = (xpos - xlong / 2)
                xhi = (xpos + xlong / 2)

                ylo = (ypos - ylong / 2)
                yhi = (ypos + ylong / 2)

                xlo = int(xlo)
                xhi = int(xhi)

                ylo = int(ylo)
                yhi = int(yhi)

                if (xlo < 1):

                    xlo = 1

                if (xhi > ncol):

                    xhi = ncol

                if (ylo < 1):

                    ylo = 1

                if (yhi > nrow):

                    yhi = nrow

                img[ylo - 1:yhi, xlo - 1:xhi] = val

    hdu[0].data = img
    hdu.writeto(maskimage, overwrite=True)
    hdu.close()

    return True




def MakeImage(newfits, sizex, sizey):
    "create a new blank Image"
# k Check
    if os.path.isfile(newfits):
        print("{} deleted; a new one is created \n".format(newfits))
    hdu = fits.PrimaryHDU()
    hdu.data = np.zeros((sizey, sizex))
    hdu.writeto(newfits, overwrite=True)

    return True


def CatArSort(SexCat,scale,offset,SexArSort,NCol,NRow):
    # k Check

    # sort the sextractor
    # catalog by magnitude,
    # get sizes for objects
    # and write it in a new file

    # The sextractor catalog must contain the following parameters:
    #   1 NUMBER                 Running object number
    #   2 ALPHA_J2000            Right ascension of barycenter (J2000)                      [deg]
    #   3 DELTA_J2000            Declination of barycenter (J2000)                          [deg]
    #   4 X_IMAGE                Object position along x                                    [pixel]
    #   5 Y_IMAGE                Object position along y                                    [pixel]
    #   6 MAG_APER               Fixed aperture magnitude vector                            [mag]
    #   7 KRON_RADIUS            Kron apertures in units of A or B
    #   8 FLUX_RADIUS            Fraction-of-light radii                                    [pixel]
    #   9 ISOAREA_IMAGE          Isophotal area above Analysis threshold                    [pixel**2]
    #  10 A_IMAGE                Profile RMS along major axis                               [pixel]
    #  11 ELLIPTICITY            1 - B_IMAGE/A_IMAGE
    #  12 THETA_IMAGE            Position angle (CCW/x)                                     [deg]
    #  13 BACKGROUND             Background at centroid position                            [count]
    #  14 CLASS_STAR             S/G classifier output
    #  15 FLAGS                  Extraction flags


    print("Sorting and getting sizes for objects \n")

    n, alpha, delta, xx, yy, mg, kr, fluxrad, ia, ai, e, theta, bkgd, idx, flg = np.genfromtxt(
        SexCat, delimiter="", unpack=True)

    n = n.astype(int)
    flg = flg.astype(int)

    Rkron = scale * ai * kr + offset

    Rwsky = scale * ai * kr + 10  + 20


#   considering to use only  KronScale instead of SkyScale
#    Rwsky = parvar.KronScale * ai * kr + parvar.Offset + parvar.SkyWidth

    Bim = (1 - e) * Rkron

    Area = np.pi * Rkron * Bim *(-1)

    (sxmin, sxmax, symin, symax) = GetSize(xx, yy, Rkron, theta, e, NCol, NRow)

    (sxsmin, sxsmax, sysmin, sysmax) = GetSize(xx, yy, Rwsky, theta, e, NCol,NRow)


    f_out = open(SexArSort, "w")

    index = Area.argsort()
    for i in index:

        line = "{} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {} {}\n".format(n[i], alpha[i], delta[i], xx[i], yy[i], mg[i], kr[i], fluxrad[i], ia[i], ai[i], e[i], theta[i], bkgd[i], idx[i], flg[i], np.int(
            np.round(sxmin[i])), np.int(np.round(sxmax[i])), np.int(np.round(symin[i])), np.int(np.round(symax[i])), np.int(np.round(sxsmin[i])), np.int(np.round(sxsmax[i])), np.int(np.round(sysmin[i])), np.int(np.round(sysmax[i])))

        f_out.write(line)

    f_out.close()

    return len(n)


def GetSize(x, y, R, theta, ell, ncol, nrow):
    "this subroutine get the maximun"
    "and minimim pixels for Kron and sky ellipse"
    # k Check
    q = (1 - ell)
    bim = q * R

    theta = theta * (np.pi / 180)  # rads!!

    # getting size

    xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                       (bim**2) * (np.sin(theta))**2)

    ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)

    ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                       (bim**2) * (np.cos(theta))**2)



    mask = xmin < 1
    if mask.any():
        xmin[mask] = 1

    mask = xmax > ncol
    if mask.any():
        xmax[mask] = ncol

    mask = ymin < 1
    if mask.any():
        ymin[mask] = 1

    mask = ymax > nrow
    if mask.any():
        ymax[mask] = nrow

    xmin = np.rint(xmin)
    ymin = np.rint(ymin)
    xmax = np.rint(xmax)
    ymax = np.rint(ymax)

    return (xmin.astype(int), xmax.astype(int), ymin.astype(int), ymax.astype(int))


def EraseObjectMask(MaskFile,tempMask,obj,fill = 0):

    hdumask = fits.open(MaskFile)
    data = hdumask[0].data

    mask = data == obj

    data[mask] = fill # removing object from mask
    
    hdumask[0].data = data

    hdumask.writeto(tempMask, overwrite=True)
    
    hdumask.close()
    
def EraseObjectMask2(maskimg,obj):

    tempmask=maskimg.copy()

    mask = tempmask == obj

    tempmask[mask] = 0 # removing object from mask
   
    return tempmask


def MakeStamps(image, catalog, maskimage, stretch, skyoff, dpi, 
                cmap, scale, offset, bright, contrast, galclass):


    hdu = fits.open(image)
    img = hdu[0].data
    hdu.close()


    hdu = fits.open(maskimage)
    mask = hdu[0].data
    hdu.close()


    NCol, NRow = GetAxis(image)


    wcs = GetWCS(image)
    #counts = GetCounts(image)


    STRETCH_CONST = stretch #for stamps sizes

    flagsat=4      ## flag value when object is saturated (or close to)

    #creates a object image: same image but counts = 0 for regions where 
    #there is no objets
    objimage = MakeObjImg(img,mask) 



    N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Class,Flg=np.genfromtxt(catalog,delimiter="",unpack=True)


    if not os.path.exists("stamps"):
        print("Creating directory for stamps ... ")
        os.makedirs("stamps")



    Rkron = scale * Ai * Kr + offset

    maskron = Rkron == 0 
    if maskron.any():
        Rkron[maskron] = 1


    Rkron = Rkron * STRETCH_CONST    

    q = (1 - E)
    bim = q * Rkron


    (xmin, xmax, ymin, ymax) = GetSize(X, Y, Rkron, Theta, E, NCol, NRow) #size of every object


    num = 0

    num_stamps = 0
    for idx, item in enumerate(N):


        check = CheckFlag(Flg[idx],flagsat)  ## check if object doesn't has saturated regions


        if  ((check == False) and (Class[idx] < galclass)):


            objimg = objimage.copy() # a new objimg for every new stamp 


            objmask = N[idx] == mask
            objimg[objmask] = 0 # removing main object from object image 
     
            objmask2 = (mask != N) & (mask != 0)

            # removing background for all the objects in the object image  
            objimg[objmask2] = objimg[objmask2] - Bkgd[idx] 

            # removes all the objects except the main target.
            stamp = img - objimg 

            # the line below is incorrect but it helps to increase the contrast
            stamp[objmask] = stamp[objmask] - Bkgd[idx] #consider to remove 

            imgstmp = "obj-" + str(round(N[idx])) + ".png"


            yy = int(X[idx]) - xmin[idx]  #interchange because numpy arrays 

            xx = int(Y[idx]) - ymin[idx] 

            #if quick:

            ShowImg(stamp[ymin[idx]-1:ymax[idx]-1,xmin[idx]-1:xmax[idx]-1], 
                        xx, yy, wcs, imgstmp, dpival = dpi, sky = Bkgd[idx], 
                        cmap = cmap, bri = bright, con = contrast)

            #slow routine

            #GetPng(stamp[ymin[idx]-1:ymax[idx]-1,xmin[idx]-1:xmax[idx]-1], 
            #        counts, wcs, dpi=dpi, cmap = cmap, namepng = imgstmp, 
            #        bri = bright, con = contrast)

            #move to folder change this for an function o os library
            runcmd = "mv  {}  stamps/{}".format(imgstmp,imgstmp)
            errmv = sp.run([runcmd], shell=True, stdout=sp.PIPE,
                               stderr=sp.PIPE, universal_newlines=True)

            num_stamps +=1
        else:

            num +=1

    print ("{} objects rejected because they have saturated pixels or are classified as stars  \n".format(num))
    print ("{} stamps created  \n".format(num_stamps))


def MakeObjImg(image,mask):
    """make object image """
    objimg = image.copy()

    zero_pix = mask == 0 

    objimg[zero_pix]  = 0

    return objimg


def ShowImg(img: np.array ,xc: int, yc: int, wcs, namepng="obj.png", 
            dpival=100, sky=1, cmap='viridis', bri = 33, con = 0.98):
    """This routine shows the image"""

        
    #hdu = fits.open(cubeimg)
    #data = (hdu[1].data.copy()).astype(float)
    #hdu.close()

    data=img

    root_ext = os.path.splitext(namepng)

    objname = root_ext[0]


    mask=data < 0 
    data[mask] = 1 # avoids problems in log
     
    fig, ax1 = plt.subplots(figsize=(7, 7)) 
    fig.subplots_adjust(left=0.04, right=0.98, bottom=0.02, top=0.98)

    ax1=fig.add_subplot(projection=wcs)

    flatdata=data.flatten()  

    flatdata.sort()


    tot=len(flatdata)

    top=round(.9*tot)
    bot=round(.1*tot)

    imgpatch=flatdata#[bot:top]

    galmin = np.min(imgpatch)
    galmin = sky 
    galmax = np.max(imgpatch)

    #if frac  < 1:
    #    galmin = (1-frac)*galmin 
    #    galmax = frac*galmax


    #if (galmin > galmax):
    #    galmin, galmax = galmax, galmin



    #my routine
    ax1.imshow(con*data+bri, origin = 'lower', norm
                = colors.LogNorm(vmin = galmin, vmax = galmax), cmap = cmap)


    ax1.set_xlabel('Right Ascension')
    ax1.set_ylabel('Declination')
    ax1.grid(color='black', ls='solid', alpha=0.1)

    ax1.set_title(objname)
    plt.savefig(namepng,dpi=dpival)
     
    plt.close()


def GetPng(data, counts, wcs, dpi=200, cmap='gray_r',namepng="obj.png", bri = 33, con = 0.98):
    "Converts image into a PNG image with axis coordinates, inverted colormap, log/zmax style"

    #Slow routine, it was avoided

    #filename = get_pkg_data_filename(Image)

    #hdu = fits.open(filename)[0]

    n, bins, patches = plt.hist(counts, bins=512, range=(min(counts), max(counts)), color='black')
    n_max = np.argsort(n)[::-1]
    n_idx = n_max[0:20]
    lim_min = min(bins[n_idx])
    lim_max = max(bins[n_idx])

    #bri = 33 # brightness, source: docs.opencv.org/3.4/d3/dc1/tutorial_basic_linear_transform.html
    #con = 0.98 # contrast, > 0

    plt.subplot(projection=wcs)
    plt.imshow(con*data+bri, cmap=cmap, norm=LogNorm(lim_min, lim_max))

    plt.xlabel('Right Ascension')
    plt.ylabel('Declination')
    plt.grid(color='black', ls='solid', alpha=0.1)
    plt.title(namepng)
    plt.tight_layout()
    #plt.savefig('%s.png' % (Image), bbox_inches='tight', dpi=dpi)
    plt.savefig(namepng, bbox_inches='tight', dpi=dpi)





#! /usr/bin/env python

import numpy as np

from astropy.io import fits

import os


class SatBox:

    def __init__(self, params: object ):
        """
        This routine creates a Ds9 box region file where it contains the saturated 
        regions of the image.
        """




        if (params.run1 == 1 and params.run2 == 1):

            print ("creating {0} for ds9 ....\n".format(params.satfileout))
            self.Ds9SatBox(params.image,params.satfileout,params.output,params.satscale,params.satoffset,params.satlevel,params.minsatsize,params.satmethod,params.satq) 

            # crea archivo  de salida de reg
            print ("recomputing flags on objects which are inside saturated regions  ....\n")
            self.putflagsat(params.output,params.output2,params.satfileout)


        elif(params.run1 ==1):

            print ("creating {0} for ds9 ....\n".format(params.satfileout))
            self.Ds9SatBox(params.image,params.satfileout,"hot.cat",params.satscale,params.satoffset,params.satlevel,params.minsatsize,params.satmethod,params.satq) 

            # crea archivo  de salida de reg
            print ("recomputing flags on objects which are inside saturated regions  ....\n")
            self.putflagsat("hot.cat","hot2.cat",params.satfileout)

            os.rename("hot2.cat","hot.cat")



        elif(params.run2 == 1):

            print ("creating {0} for ds9 ....\n".format(params.satfileout))
            self.Ds9SatBox(params.image,params.satfileout,"cold.cat",params.satscale,params.satoffset,params.satlevel,params.minsatsize,params.satmethod,params.satq) 

            # crea archivo  de salida de reg
            print ("recomputing flags on objects which are inside saturated regions  ....\n")
            self.putflagsat("cold.cat","cold2.cat",params.satfileout)

            os.rename("cold2.cat","cold.cat")






    def Ds9SatBox (self,image,satfileout,sexcat,satscale,satoffset,satlevel,minsatsize,method,satq=0.5):
        "Creates a file for ds9 which selects bad saturated regions"


        (xx,yy,sx,sy,N)=self.GetSatBox(image,satfileout,sexcat,satlevel,minsatsize,satq)

        ### obtains maximun size
        (imaxx, imaxy) = self.GetAxis(image)

        #corrected for python array
        #imaxx= imaxx - 1
        #imaxy= imaxy - 1



        # increasing size of sat regions 
        sx = sx * satscale + satoffset
        sy = sy * satscale + satoffset


        (xx,yy,sx,sy,N)=self.AbsorbSmallBox(xx,yy,sx,sy,N) #remove small regions inside big ones 
        (xx,yy,sx,sy,N)=self.ResizeBox(xx,yy,sx,sy,N) #readjust size for neighbors regions
        #(xx,yy,sx,sy,N)=self.RemoveDupBox(xx,yy,sx,sy,N) #remove this one?
        (xx,yy,sx,sy,N)=self.AbsorbSmallBox(xx,yy,sx,sy,N) # final touch 

        
        f_out = open(satfileout, "w")

        for idx, item in enumerate(N):


            sidex=sx[idx]
            sidey=sy[idx]
            xcent=xx[idx]
            ycent=yy[idx]

            # increasing sides of saturated regions

            #sidex = sidex * satscale + satoffset
            #sidey = sidey * satscale + satoffset


            divflag = False
            
            #divides in two: horizontal and vertical box if axis ratio < satq
            if sidex >= sidey: #horizontal
                div = sidey/sidex
                if div <= satq:
                    divflag = True
                    sidey2=sidex*(satq) #reduction factor
                    sidex2=sidey
            else: #vertical
                div = sidex/sidey
                if div <= satq:
                    divflag = True
                    sidey2=sidex
                    sidex2=sidey*(satq) #reduction factor

            xmin,xmax,ymin,ymax = self.BoxSide2Corners(xcent, ycent, sidex, sidey)
            xmin,xmax,ymin,ymax = self.CorrectCorners(xmin,xmax,ymin,ymax,imaxx,imaxy)
            xcent, ycent, sidex, sidey  = self.BoxCorners2Side(xmin,xmax,ymin,ymax)


            line="box({0},{1},{2},{3},0) # color=red move=1 \n".format(xcent,ycent,sidex,sidey)
            f_out.write(line)


            if divflag:


               # here we have 4 methods for saturated regions divided in one vertical and one horizontal 
               # select the best method for your convinience. However the best method so far 
               # has been the third one, so this is one for default

                #method = 4
                
                posxmax,posymax = self.GetMaxCor(image,xmin,xmax,ymin,ymax)
                posxmax2,posymax2 = self.GetMaxCor2(image,xmin,xmax,ymin,ymax) # for method 4

                xmin,xmax,ymin,ymax = self.BoxSide2Corners(xcent, ycent, sidex2, sidey2)
                xmin,xmax,ymin,ymax = self.CorrectCorners(xmin,xmax,ymin,ymax,imaxx,imaxy)
 
                #method 1
                if method == 1:
                    # 5a. test moving the center of the new box or
                   xmin,xmax,ymin,ymax = self.BoxCenterInPoint(posxmax,posymax,xmin,xmax,ymin,ymax)
                    ###
                    
                #method 2
                elif method ==2:
                    # 5b. test increazing size of box.
                    xmin,xmax,ymin,ymax = self.IncludePointInBox(posxmax,posymax,xmin,xmax,ymin,ymax)

                #method 3
                elif method ==3:
                    #5c. combining methods
                    xmin,xmax,ymin,ymax = self.IncludeBoxCenterInPoint(posxmax,posymax,xmin,xmax,ymin,ymax)

                    ### 

                #method 4
                elif method ==4:
                    #5c. combining methods and avoids to select the greatest negative pixexl value as a center

                    xmin,xmax,ymin,ymax = self.IncludeBoxCenterInPoint(posxmax2,posymax2,xmin,xmax,ymin,ymax)

 


                xmin,xmax,ymin,ymax = self.CorrectCorners(xmin,xmax,ymin,ymax,imaxx,imaxy)
                xcent, ycent, sidex2, sidey2  = self.BoxCorners2Side(xmin,xmax,ymin,ymax)

                ##########

                line2="point({0},{1}) # color=red point=boxcircle font=\"times 10 bold\" text={{ {2} }} \n".format(xcent,ycent,np.int32(N[idx]))

                f_out.write(line2)


                line="box({0},{1},{2},{3},0) # color=red move=1 \n".format(xcent,ycent,sidex2,sidey2)
                f_out.write(line)


        f_out.close()



    def IncludePointInBox(self,xx,yy,xmin,xmax,ymin,ymax):

            pflag=self.IsPointInBox(xx,yy,xmin,xmax,ymin,ymax)

            if not(pflag):

                if (xx <= xmin):
                    xmin = xx - 4 

                if (yy <= ymin):
                    ymin = yy - 4

                if (xx >= xmax):
                    xmax = xx + 4

                if (yy >= ymax):
                    ymax = yy + 4


            return xmin,xmax,ymin,ymax

    def BoxCenterInPoint(self,xx,yy,xmin,xmax,ymin,ymax):



            pflag=self.IsPointInBox(xx,yy,xmin,xmax,ymin,ymax)

            if not(pflag):

                xc, yc, sx,sy  = self.BoxCorners2Side(xmin,xmax,ymin,ymax)

                xc2 = xx
                yc2 = yy


                (xmin,xmax,ymin,ymax)=self.BoxSide2Corners(xc2,yc2,sx,sy)


            return xmin,xmax,ymin,ymax


    def IncludeBoxCenterInPoint(self,xx,yy,xmin,xmax,ymin,ymax):
            '''combines the two methods'''

            pflag=self.IsPointInBox(xx,yy,xmin,xmax,ymin,ymax)

            if not(pflag):

                if (xx <= xmin):
                    xmin = xx - 4 

                if (yy <= ymin):
                    ymin = yy - 4

                if (xx >= xmax):
                    xmax = xx + 4

                if (yy >= ymax):
                    ymax = yy + 4

                xc, yc, sx,sy  = self.BoxCorners2Side(xmin,xmax,ymin,ymax)

                xc2 = xx
                yc2 = yy

                (xmin,xmax,ymin,ymax)=self.BoxSide2Corners(xc2,yc2,sx,sy)



            return xmin,xmax,ymin,ymax





    def GetMaxCor(self,image,xmin,xmax,ymin,ymax):
            '''Get coordinate (x,y) where the max value in counts'''
        
            hdu = fits.open(image)
            imgdat = hdu[0].data
            hdu.close()

            chunkimg = np.abs(imgdat[ymin-1:ymax-1,xmin-1:xmax-1])

            mask = chunkimg < 0
            chunkimg[mask]=chunkimg[mask]*-1

            p = np.where(chunkimg == np.amax(chunkimg))

            return  p[1][0] + xmin, p[0][0] + ymin
 
    def GetMaxCor2(self,image,xmin,xmax,ymin,ymax):
            '''Get coordinate (x,y) where the max value in counts. Avoids negative pixels'''
        
            hdu = fits.open(image)
            imgdat = hdu[0].data
            hdu.close()

            chunkimg = imgdat[ymin-1:ymax-1,xmin-1:xmax-1]

            #mask = chunkimg < 0
            #chunkimg[mask]=chunkimg[mask]*-1

            p = np.where(chunkimg == np.amax(chunkimg))

            return  p[1][0] + xmin, p[0][0] + ymin
 



    def ResizeBox(self,xx,yy,sx,sy,N):

        xc, yc, sidx, sidy = np.array([]), np.array([]), np.array([]), np.array([])
        nn=np.array([])


        for idx, item in enumerate(N):

            xx1,yy1,sx1,sy1,nn1 = xx[idx],yy[idx],sx[idx],sy[idx],N[idx]
            erflag = True

            (xmin,xmax,ymin,ymax)=self.BoxSide2Corners(xx1,yy1,sx1,sy1)


            for idx2, item2 in enumerate(N):

                xx2,yy2,sx2,sy2 = xx[idx2],yy[idx2],sx[idx2],sy[idx2] # remove this line after tests

                if idx!=idx2:

                    (xmin2,xmax2,ymin2,ymax2)=self.BoxSide2Corners(xx2,yy2,sx2,sy2)
                    
                    pflag=self.IsPointInBox(xx2,yy2,xmin,xmax,ymin,ymax)

                    if pflag:

                        if (xmin2 <= xmin):
                            xmin = xmin2 - 4 

                        if (ymin2 <= ymin):
                            ymin = ymin2 - 4

                        if (xmax2 >= xmax):
                            xmax = xmax2 + 4

                        if (ymax2 >= ymax):
                            ymax = ymax2 + 4


                        xx1, yy1, sx1,sy1  = self.BoxCorners2Side(xmin,xmax,ymin,ymax)
                        #update new size in old array 
                        xx[idx],yy[idx],sx[idx],sy[idx] =  xx1, yy1, sx1,sy1

            xc=np.append(xc,xx1)
            yc=np.append(yc,yy1)
            sidx=np.append(sidx,sx1)
            sidy=np.append(sidy,sy1)
            nn=np.append(nn,nn1)

        return(xc,yc,sidx,sidy,nn)

    def AbsorbSmallBox(self,xx,yy,sx,sy,N):
        "Removes small boxes inside big boxes"
        xc, yc, sidx, sidy = np.array([]), np.array([]), np.array([]), np.array([])
        nn=np.array([])


        for idx, item in enumerate(N):

            xx1,yy1,sx1,sy1,nn1 = xx[idx],yy[idx],sx[idx],sy[idx],N[idx]
            foflag = False 

            for idx2, item2 in enumerate(N):

                xx2,yy2,sx2,sy2 = xx[idx2],yy[idx2],sx[idx2],sy[idx2]

                if idx!=idx2:
                    
                    bflag=self.IsBoxInBox(xx1,yy1,sx1,sy1,xx2,yy2,sx2,sy2)


                    if bflag == True:
                        foflag = True
                        break


            if foflag == False:

                xc=np.append(xc,xx1)
                yc=np.append(yc,yy1)
                sidx=np.append(sidx,sx1)
                sidy=np.append(sidy,sy1)
                nn=np.append(nn,nn1)


        return(xc,yc,sidx,sidy,nn)

    def RemoveDupBox(self,xx,yy,sx,sy,N):
        "Removes duplicated boxes and small boxes inside big boxes"

        xc, yc, sidx, sidy = np.array([]), np.array([]), np.array([]), np.array([])
        nn=np.array([])


        for idx, item in enumerate(N):

            xx1,yy1,sx1,sy1,nn1 = xx[idx],yy[idx],sx[idx],sy[idx],N[idx]

            foflag = False 

            for idx2, item2 in enumerate(nn):

                xx2,yy2,sx2,sy2 = xc[idx2],yc[idx2],sidx[idx2],sidy[idx2]
                    
                bflag=self.AreTwoBoxSame(xx1,yy1,sx1,sy1,xx2,yy2,sx2,sy2)

                if (bflag == True):
                    foflag = True
                    break


            if foflag == False:

                xc=np.append(xc,xx1)
                yc=np.append(yc,yy1)
                sidx=np.append(sidx,sx1)
                sidy=np.append(sidy,sy1)
                nn=np.append(nn,nn1)


        return(xc,yc,sidx,sidy,nn)




    def BoxSide2Corners(self,X,Y,sidex,sidey):

             # -1 correction for python array
            #xmin= X - sidex / 2 - 1
            #xmax= X + sidex / 2  -1
            #ymin= Y - sidey / 2  -1
            #ymax= Y + sidey / 2  -1

            # -1 correction for python array
            xmin= X - sidex / 2 
            xmax= X + sidex / 2
            ymin= Y - sidey / 2
            ymax= Y + sidey / 2

            xmin=np.int32(np.round(xmin))
            xmax=np.int32(np.round(xmax))
            ymin=np.int32(np.round(ymin))
            ymax=np.int32(np.round(ymax))


            return (xmin,xmax,ymin,ymax)


    def CorrectCorners(self,xmin,xmax,ymin,ymax,imaxx,imaxy):


            # correct if xmin, xmax, ymin, ymax have values outsides of image limits

            if xmin < 1:
                xmin=1
            if xmax > imaxx:
                xmax=imaxx

            if ymin < 1:
                ymin=1
            if ymax > imaxy:
                ymax=imaxy

            return (xmin,xmax,ymin,ymax)



    def BoxCorners2Side(self,xmin,xmax,ymin,ymax):

            sidex= xmax - xmin
            sidey= ymax - ymin

            # increase +1 for conversion to DS9
            #xcent = xmin + sidex/2 + 1
            #ycent = ymin + sidey/2 + 1

            xcent = xmin + sidex/2 
            ycent = ymin + sidey/2


            xcent=np.int32(np.round(xcent))
            ycent=np.int32(np.round(ycent))
    

            return (xcent,ycent,sidex,sidey)


    def putflagsat(self,sexfile,sexfile2,regfile):
        "Put flags on objects which are inside saturated regions"


        f_out= open(sexfile2, "w")

        scale = 1
        offset=0


        flagsat=4      ## flag value when object is saturated (or close to)


        N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(sexfile,delimiter="",unpack=True)

        for idx, item in enumerate(N):

            Rkron = scale * Ai[idx] * Kr[idx] + offset

            if Rkron == 0:

                Rkron = 1

            q = (1 - E)
            bim = q * Rkron


            check=self.CheckFlag(Flg[idx],flagsat)  ## check if object doesn't has saturated regions
    #        regflag=CheckSatReg(X[idx],Y[idx],Rkron,Theta[idx],E[idx],regfile)    ## check if object is inside of a saturaded box region indicated by user in ds9
            regflag=self.CheckSatReg2(X[idx],Y[idx],regfile)    ## check if object is inside of a saturaded box region indicated by user in ds9


            if  (check == False ) and ( regflag == True) :

                Flg[idx] = Flg[idx] + 4


                line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])

                f_out.write(line)


            else:

                line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])


                f_out.write(line)



        f_out.close()

    def GetAxis(self,Image):
        # k Check
        "Get number of rows and columns from the image"

        hdu = fits.open(Image)
        ncol = hdu[0].header["NAXIS1"]
        nrow = hdu[0].header["NAXIS2"]
        hdu.close()
        return ncol, nrow



    def CheckFlag(self,val,check):
       "Check for flag contained in $val, returns True if found "

       flag = False
       mod = 1
       fmax=128


       while (mod != 0):


           res = int(val/fmax)

           if (fmax == check and res == 1 ):

               flag=True


           mod = val % fmax

           val = mod
           fmax = fmax/2


       return flag


    def CheckSatReg2(self,x,y,filein):
       "Check if object is inside of saturated region. returns True if at least one pixel is inside"
    ## check if object is inside of
    ## saturaded region as indicated by ds9 box region
    ## returns True if object center is in saturaded region

       flag = False

       with open(filein) as f_in:

           lines = (line.rstrip() for line in f_in) # All lines including the blank ones
           lines = (line.split('#', 1)[0] for line in lines) # remove comments
           lines = (line.rstrip() for line in lines)   # remove lines containing only comments
           lines = (line for line in lines if line) # Non-blank lines

           for line in lines:

               if (line != "image"):

                   (box,info)=line.split('(')

                   if(box == "box"):

                       (xpos,ypos,xlong,ylong,trash)=info.split(',')

                       xpos=float(xpos)
                       ypos=float(ypos)
                       xlong=float(xlong)
                       ylong=float(ylong)


                       xlo = xpos - xlong/2
                       xhi = xpos + xlong/2

                       ylo = ypos - ylong/2
                       yhi = ypos + ylong/2

                       if ( (x > xlo and x < xhi) and (y > ylo and y < yhi) ):
                           flag=True
                           break


       return flag


    def GetSatBox (self,image,satfileout,sexcat,satlevel,minsatsize,satq):
        "Obtains the saturated boxes from the sextractor catalog "


        flagsat=4      ## flag value when object is saturated (or close to)


        ### read image
        hdu = fits.open(image)
        imgdat = hdu[0].data
        hdu.close()

        ### obtains maximun size
        (imaxx, imaxy) = self.GetAxis(image)

        #corrected for python array
        #imaxx= imaxx -1
        #imaxy= imaxy -1


        f_out = open(satfileout, "w")

        N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(sexcat,delimiter="",unpack=True)


        line="image \n"
        f_out.write(line)


        xx, yy, sx, sy = np.array([]), np.array([]), np.array([]), np.array([])
        nn=np.array([])

        for idx, item in enumerate(N):


            check=self.CheckFlag(Flg[idx],flagsat)  # check if object has saturated region

            if (check):

                bi=Ai[idx]*(1-E[idx])

                Theta[idx] = Theta[idx] * np.pi /180  #rads!!!

                Rkronx =  2 * Ai[idx] * Kr[idx] * np.cos(Theta[idx])
                Rkrony =  2 * bi * Kr[idx] * np.sin(Theta[idx])

                if Rkronx <= minsatsize:
                    Rkronx = minsatsize

                if Rkrony <= minsatsize:
                    Rkrony = minsatsize

                Rkronx=np.int32(np.round(Rkronx))
                Rkrony=np.int32(np.round(Rkrony))


                xmin,xmax,ymin,ymax = self.BoxSide2Corners(X[idx],Y[idx], Rkronx, Rkrony)
                xmin,xmax,ymin,ymax = self.CorrectCorners(xmin,xmax,ymin,ymax,imaxx,imaxy) #correct corners
                #chunkimg = imgdat[ymin:ymax,xmin:xmax]
                chunkimg = imgdat[ymin-1:ymax-1,xmin-1:xmax-1]


                satm= (chunkimg > satlevel) + (chunkimg < (-1)*satlevel)
                satind= np.where(satm)

                y,x=satind

                errmsg="Can't found sat pixels in [{}:{},{}:{}]. Try to increase MinSatSize value \n".format(xmin+1,xmax+1,ymin+1,ymax+1)
                assert x.size != 0, errmsg
                
                #return to original coordinates
                #satymin = y.min() + ymin
                #satxmin = x.min() + xmin
                #satymax = y.max() + ymin
                #satxmax = x.max() + xmin

                #return to original coordinates
                satymin = y.min() + ymin + 1
                satxmin = x.min() + xmin + 1
                satymax = y.max() + ymin + 1
                satxmax = x.max() + xmin + 1

               
                xcent, ycent, sidex, sidey  = self.BoxCorners2Side(satxmin,satxmax,satymin,satymax)


                xx=np.append(xx,xcent)
                yy=np.append(yy,ycent)
                sx=np.append(sx,sidex)
                sy=np.append(sy,sidey)
                nn=np.append(nn,N[idx])



        f_out.close()


        return (xx,yy,sx,sy,nn)
    

    def IsPointInBox(self,xx,yy,xmin,xmax,ymin,ymax):

        flag=False

        if ((xmin < xx < xmax ) and ( ymin < yy < ymax)):
            flag = True


        return flag


    def IsBoxInBox(self,xx1,yy1,sx1,sy1,xx2,yy2,sx2,sy2):
        "Check if box is inside another box"
        flag=False


        (xmin,xmax,ymin,ymax)=self.BoxSide2Corners(xx1,yy1,sx1,sy1)
        (xmin2,xmax2,ymin2,ymax2)=self.BoxSide2Corners(xx2,yy2,sx2,sy2)


    
        if ((xmin2 < xmin) and (xmax < xmax2) ):
            if ((ymin2 < ymin) and (ymax < ymax2) ):
                flag = True
 
        return flag


    def AreTwoBoxSame(self,xx1,yy1,sx1,sy1,xx2,yy2,sx2,sy2):
        "Check if two boxes are the same"

        flag=False


        if ((xx1  == xx2 ) and (yy1  == yy2) and (sx1  == sx2 ) and (sy1  == sy2)): 
                flag = True

        return flag








#! /usr/bin/env python

import sys
import subprocess as sp



def wsex(params):

    # hot.sex

    flaghead = False

    f_out = open(params.outhot, "w")

    with open(params.sexfile) as f_in2:

        lines = (line.rstrip() for line in f_in2) # All lines including the blank ones
        lines = (line.split('#', 1)[0] for line in lines) # remove comments
        lines = (line.rstrip() for line in lines)   # remove lines containing only comments
        lines = (line for line in lines if line) # Non-blank lines

        for line2 in lines:

            (linparams)=line2.split()

            if linparams[0] ==  "MAG_ZEROPOINT":
	            line2= "MAG_ZEROPOINT "+str(params.zpt)
            if linparams[0] ==  "GAIN":
	            line2= "GAIN "+str(params.gain)
            if linparams[0] ==  "PIXEL_SCALE":
	            line2= "PIXEL_SCALE "+str(params.plate)
            if linparams[0] ==  "SEEING_FWHM":
	            line2= "SEEING_FWHM "+str(params.seeing)



            if linparams[0] == "DEBLEND_NTHRESH":
                line2="DEBLEND_NTHRESH "+str(params.dn1)

            if linparams[0] == "DEBLEND_MINCONT":
                line2= "DEBLEND_MINCONT "+ str(params.dm1)

            if linparams[0] ==  "ANALYSIS_THRESH":
                line2= "ANALYSIS_THRESH "+str(params.at1)

            if linparams[0] ==  "DETECT_THRESH":
   	            line2= "DETECT_THRESH "+str(params.dt1)

            if linparams[0] ==  "DETECT_MINAREA":
	            line2= "DETECT_MINAREA "+str(params.da1)

            if linparams[0] ==  "CATALOG_NAME":
	            line2= "CATALOG_NAME hot.cat"

            if linparams[0] ==  "BACK_SIZE":
	            line2= "BACK_SIZE "+str(params.bs1)

            if linparams[0] ==  "BACK_FILTERSIZE":
	            line2= "BACK_FILTERSIZE "+str(params.bf1)

            if linparams[0] ==  "CATALOG_TYPE":
                if linparams[1] !=  "ASCII":
	                   flaghead = True



            line2 = line2+"\n"
            f_out.write(line2)


    f_out.close()


    # cold.sex

    f_out = open(params.outcold, "w")

    with open(params.sexfile) as f_in2:

        lines = (line.rstrip() for line in f_in2) # All lines including the blank ones
        lines = (line.split('#', 1)[0] for line in lines) # remove comments
        lines = (line.rstrip() for line in lines)   # remove lines containing only comments
        lines = (line for line in lines if line) # Non-blank lines

# every line of the file:
        for line2 in lines:

            (linparams)=line2.split()

            if linparams[0] ==  "MAG_ZEROPOINT":
	            line2= "MAG_ZEROPOINT "+str(params.zpt)
            if linparams[0] ==  "GAIN":
	            line2= "GAIN "+str(params.gain)
            if linparams[0] ==  "PIXEL_SCALE":
	            line2= "PIXEL_SCALE "+str(params.plate)
            if linparams[0] ==  "SEEING_FWHM":
	            line2= "SEEING_FWHM "+str(params.seeing)


            if linparams[0] == "DEBLEND_NTHRESH":
                line2="DEBLEND_NTHRESH "+str(params.dn2)

            if linparams[0] == "DEBLEND_MINCONT":
                line2= "DEBLEND_MINCONT "+ str(params.dm2)

            if linparams[0] ==  "ANALYSIS_THRESH":
                line2= "ANALYSIS_THRESH "+str(params.at2)

            if linparams[0] ==  "DETECT_THRESH":
   	            line2= "DETECT_THRESH "+str(params.dt2)

            if linparams[0] ==  "DETECT_MINAREA":
	            line2= "DETECT_MINAREA "+str(params.da2)

            if linparams[0] ==  "CATALOG_NAME":
	            line2= "CATALOG_NAME cold.cat"

            if linparams[0] ==  "BACK_SIZE":
	            line2= "BACK_SIZE "+str(params.bs2)

            if linparams[0] ==  "BACK_FILTERSIZE":
	            line2= "BACK_FILTERSIZE "+str(params.bf2)


            f_out.write(line2+"\n")


    f_out.close()

    if flaghead:
        print ("CATALOG_TYPE must be ASCII in default.sex. Ending program.. \n") # change here to assert function
        sys.exit()


def runsex(params):

######  Running Sextractor  #######

    if (params.run1 == 1):
        print("Running hot.sex ")
        print("sex -c {} {} \n".format(params.outhot,params.image))
        runcmd="sex -c {} {} ".format(params.outhot,params.image)
        # Run GALFIT
        err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True)  
        if err.returncode != 0:
            print("error: ",err.returncode,err.stderr)
            sys.exit("There was a problem running sextractor")

    if (params.run2 == 1):
        print("Running cold.sex ")
        print("sex -c {} {} \n".format(params.outcold,params.image))
        runcmd="sex -c {} {} ".format(params.outcold,params.image)
        # Run GALFIT
        err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True) 
        if err.returncode != 0:
            print("error: ",err.returncode,err.stderr)
            sys.exit("There was a problem running sextractor")















import clusex

class Params:

    # class variables
    # default values


    image = "none.fits"


    outhot  = "hot.sex"
    outcold = "cold.sex"
    sexfile= "default.sex"
    output="hc.cat"
    output2="hotcold.cat"
    scale=1
    offset = 0
    scale2=1


    run1=run2=1

    #create = 0 #has moved to makemask


    satscale=1
    minsatsize=20
    satq=0.1
    satlevel=50000
    

    regoutfile = "hotcold.reg"

    #maskfile="mask.fits"

    SexArSort="sortar.cat"

    dn1 = dn2 = 1
    dm1 = dm2 = 1
    at1 = at2 = 1
    dt1 = dt2 = 1
    da1 = da2 = 1
    bs1 = bs2 = 1
    bf1 = bf2 = 1


    satfileout="sat.reg"


    satoffset  = 1

    satlevel = 100000

    create = 0


    output2 = "hotcold.cat"

    minsatsize = 10

    satq = 0.1


    tol = 0.5  #Fraction tolerance for differences between the two catalogs (proportion)  
    red = 0.3  # reduction factor or reduction coefficient 
    minrad = 10 # minimum radius to apply for reduction factor
    flagminrad = False # if minrad is not readed from conf file, then fwhm is set as minrad

    satmethod = 3 


    zpt = 25

    gain = 10


    plate = 1


    seeing = 2

    joinscale = 0.7


    scalecor = 1

    flagDs9 = 1 




def printWelcome():


    print("CluSex: Sextractor on Cluster Galaxies")

    print("Version:",clusex.__version__)

    url = "https://github.com/canorve/CluSex"
    
    print("webpage: "+url+"\n")


    #print("CluSex joins two sextractor catalogs. ")
    #print("It creates masks, finds saturated regions and computes sky background")







#! /usr/bin/env python

import numpy as np
import argparse
import os
import sys

from clusex.lib.check  import CheckKron

from clusex.lib.check import CheckFlag 
from clusex.lib.check import CheckSatReg2



def joinsexcat (maincat,secondcat,output,JoinScale,incFlag=False, red=0.1,minrad = 5):
    "merges two Sextractor catalogs"

    f_out = open(output, "w")

    KronScale2 = 1

    try:
    #maincat
        N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(maincat,delimiter="",unpack=True)
    except IOError:
        print("bad sex file or wrong number of columns ")
        sys.exit(1)
    AR         = 1 - E
    RKron      = JoinScale * Ai * Kr



    maskron = RKron <= 0
    RKron[maskron]=1

    maskar = AR <= 0.005

    AR[maskar]=0.005

    for idx, item in enumerate(N):

        line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])



        f_out.write(line)


    total =len(N)

    NewN=total + 1

    try: 
        #second cat
        N2,Alpha2,Delta2,X2,Y2,Mg2,Kr2,Fluxr2,Isoa2,Ai2,E2,Theta2,Bkgd2,Idx2,Flg2=np.genfromtxt(secondcat,delimiter="",unpack=True)
    except:
        print("bad sextractor file or wrong number of columns ")
        sys.exit(1)


    AR2         = 1 - E2
    RKron2      = KronScale2 * Ai2 * Kr2


    maskar2 = AR2 <= 0.005
    AR2[maskar2]=0.005


    count=0
    count2=0

    flag1 = False
    flag2 = False
 
    distmax = 5 # dist max to compare with incFlag=True

    dist = 0

    if incFlag:
        print("Including all the galaxies from the second catalog that are not in the first catalog" )
    else:
        print("Including all the galaxies from the second catalog that are outside from ellipse objects from the first catalog" )

    for idx2, item2 in enumerate(N2):

        if incFlag:
            flagf = True
            for idx, item in enumerate(N):
                
                dx = X[idx]-X2[idx2]
                dy = Y[idx]-Y2[idx2]
                dist=np.sqrt( dx**2  + dy**2 )

                if dist < distmax:   
                    flagf = False 
                    break

            if flagf:

                Kr2[idx2] = red * Kr2[idx2]

                if Kr2[idx2] * Ai2[idx2] < minrad:    

                    Kr2[idx2] = minrad/Ai2[idx2]
 

                line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(NewN, Alpha2[idx2], Delta2[idx], X2[idx2], Y2[idx2], Mg2[idx2], Kr2[idx2], Fluxr2[idx2], Isoa2[idx2], Ai2[idx2], E2[idx2], Theta2[idx2], Bkgd2[idx2], Idx2[idx2], Flg2[idx2])

                f_out.write(line)

                NewN+=1
                count2+=1
            else:
                count+=1       


        else:

            flagf =False
            for idx, item in enumerate(N):


                flag1=CheckKron(X2[idx2],Y2[idx2],X[idx],Y[idx],RKron[idx],Theta[idx],AR[idx])
                #flag2=CheckKron(X[idx],Y[idx],X2[idx2],Y2[idx2],RKron2[idx2],Theta2[idx2],AR2[idx2])
                flagf=flag1 or flag2

                if flagf:   # boolean value
                    break

            if not flagf:

                line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(NewN, Alpha2[idx2], Delta2[idx], X2[idx2], Y2[idx2], Mg2[idx2], Kr2[idx2], Fluxr2[idx2], Isoa2[idx2], Ai2[idx2], E2[idx2], Theta2[idx2], Bkgd2[idx2], Idx2[idx2], Flg2[idx2])

                f_out.write(line)

                NewN+=1
                count2+=1
            else:
                count+=1       

    f_out.close()

    linout="{} objects from second run rejected ".format(count)
    print(linout)

    linout="{} objects were added from second run ".format(count2)
    print(linout)



def putFlagSat(sexfile,sexfile2,regfile):
    """Put flags on objects which are inside saturated regions"""


    f_out= open(sexfile2, "w")

    scale = 1
    offset=0


    flagsat=4      ## flag value when object is saturated (or close to)


    N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(sexfile,delimiter="",unpack=True)

    for idx, item in enumerate(N):

        Rkron = scale * Ai[idx] * Kr[idx] + offset

        if Rkron == 0:

            Rkron = 1

        q = (1 - E)
        bim = q * Rkron


        check=CheckFlag(Flg[idx],flagsat)  ## check if object doesn't has saturated regions
    #        regflag=CheckSatReg(X[idx],Y[idx],Rkron,Theta[idx],E[idx],regfile)    ## check if object is inside of a saturaded box region indicated by user in ds9
        regflag=CheckSatReg2(X[idx],Y[idx],regfile)    ## check if object is inside of a saturaded box region indicated by user in ds9


        if  (check == False ) and ( regflag == True) :

            Flg[idx] = Flg[idx] + 4


            line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])

            f_out.write(line)


        else:

            line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])


            f_out.write(line)


    f_out.close()





#! /usr/bin/env python

#    CluSex is a set of routines that aids Sextractor 
#    to perform on images of cluster galaxies
#    Copyright (C) 2022  Christopher Añorve 
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.

#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.


import numpy as np
import sys
import os
import subprocess as sp
from astropy.io import fits
import os.path
import scipy

from pathlib import Path
import shutil
import argparse


from clusex.lib.init import Params
from clusex.lib.init import printWelcome

from clusex.lib.read import readcon 
from clusex.lib.writesex import wsex
from clusex.lib.writesex import runsex

from clusex.lib.join import joinsexcat 



from clusex.lib.ds9 import ds9kron


from clusex.lib.make import MakeMask 
from clusex.lib.make import MakeImage
from clusex.lib.make import MakeSatBox 
from clusex.lib.make import CatArSort


from clusex.lib.radcor import RadMod


from clusex.lib.satbox import SatBox 


def main():

    ######################################
    ##### Reading argument parser ########
    ######################################

    printWelcome()


    parser = argparse.ArgumentParser(description="CluSex: combines two Sextractor catalogs among other stuff")

    # required arguments
    parser.add_argument("ConfigFile",help="CluSex configuration file ")
    #parser.add_argument("image",help="FITS image file of the galaxy cluster ")  # incorporated   


    args = parser.parse_args()

    confile = args.ConfigFile 
    #image = args.image



    ######################################
    ##### initial parameters #############
    ######################################


    #copying files to actual folder

    filsex = Path(__file__).parent / "config/default.sex"
    filnnw = Path(__file__).parent / "config/default.nnw"
    filcon = Path(__file__).parent / "config/default.conv"
    filpar = Path(__file__).parent / "config/sex.param"

    to_des = Path('.')

    shutil.copy2(filsex, to_des)  
    shutil.copy2(filnnw, to_des)  
    shutil.copy2(filcon, to_des) 
    shutil.copy2(filpar, to_des)



    params=Params()

    #########################################
    ##### Reading paramter file #############
    #########################################

    readcon(params,confile)

    ####################################################
    ##### Reading and writing sextractor files #########
    ####################################################

    wsex(params)

    ##################################
    ##### Running Sextractor #########
    ##################################



    runsex(params)


    ######################################################################
    ### Correcting radius for large low-surface brightness galaxies  ##### 
    ######################################################################

    print("correcting radius of galaxies")



    if (params.run1 == 1 and params.run2 == 1):

        #incorporate this two parameters to input file
        #tol = 0.5 #tolerance for radius differences between the two catalogs (proportion) 
        #red = 0.3  # reduction factor

        if not(params.flagminrad):
            params.minrad = params.seeing


        RadMod("hot.cat","cold.cat","hot2.cat",tol=params.tol,
                red=params.red,minrad=params.minrad,scalecor=params.scalecor)
        RadMod("cold.cat","hot.cat","cold2.cat",tol=params.tol,
                red=params.red,minrad=params.minrad,scalecor=params.scalecor)



        os.rename("hot2.cat","hot.cat")
        os.rename("cold2.cat","cold.cat")

    else:

        print("Unable to correct radius because it needs to run sextractor twice. run = 1 run2 = 1 \n")

    #####################################################################
    ##### joins the two output sextractor files into one single file ####
    #####################################################################


    if (params.run1 == 1 and params.run2 == 1):
        print ("joining hot.cat and cold.cat catalogs ....\n");
        joinsexcat("hot.cat","cold.cat",params.output,params.joinscale)

    else:
        print("Unable to join catalogs because sextractor was not used \n")



    ###########################################################################
    ##### search for saturated stars and figure out the saturated area  ####### 
    ##### It also modify flags of objects near to saturated stars ############# 
    ##########################################################################
        

    SatBox(params)
    

    ##################################################################
    ###### creates the output files for DS9 ###################### 
    ##################################################################

    if params.flagDs9 == 1:


        if (params.run1 == 1 and params.run2 == 1):
            print ("{0} is the output catalog  ....\n".format(params.output2))
            print ("Creating ds9 check region file....\n")
            ds9kron(params.output2,params.regoutfile,params.scale,params.offset)
        elif(params.run1 ==1):
            print ("{0} is the output catalog  ....\n".format("hot.cat"))
            print ("Creating ds9 check region file....\n")
            ds9kron("hot.cat",params.regoutfile,params.scale,params.offset)
        elif(params.run2==1):
            print ("{0} is the output catalog  ....\n".format("cold.cat"))
            print ("Creating ds9 check region file....\n")
            ds9kron("cold.cat",params.regoutfile,params.scale,params.offset)


        print ("Running ds9 ...\n")
        runcmd="ds9 -tile column -cmap grey -invert -log -zmax -regions shape box {} -regions {} -regions {} ".format(params.image,params.regoutfile,params.satfileout)
        err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True)  


    else:

        print("CluSex won't run ds9 because DisplayDs9 is set to 0 ")


    #####################################################################
    ###### Makemask has been moved to an independient routine ###########
    #####################################################################


    print("CluSex has finished.")






    #############################################################################
    ######################### End of program  ######################################


#     ______________________________________________________________________
#    /___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/___/_/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/|
#   |___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|__/|
#   |_|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|___|/




#end of program
if __name__ == '__main__':
    main()



#! /usr/bin/env python

#from ellipsect.lib.libs import *

#from ellipsect import *

import numpy as np 


class SkyCal:
    "This class compute the sky using two methods: random boxes and sky gradient"

    def RandBox(self,ImageFile,MaskFile,xx,yy,thetadeg,q,Rinit,box,num,Rmax,outliers=True):
        "random box method to compute sky"

        self.xx = xx 
        self.yy = yy

        self.thetadeg=90 + thetadeg

        self.q = q
        self.Rinit = Rinit
       
        self.box = box 
        self.num = num

        self.outliers = outliers

        ###

        #hdumask = fits.open(MaskFile)
        #self.maskimg = hdumask[0].data
        #hdumask.close()

        self.maskimg = MaskFile 


        #hdu=fits.open(ImageFile)
        #self.img = hdu[0].data
        #hdu.close()

        self.img = ImageFile 


        ####
        
        (self.nrow,self.ncol)=self.img.shape

        xmin,xmax,ymin,ymax,Rend = self.GetXYRBorder()

        if Rmax != 0:
            if Rmax <= Rend:
                Rend = Rmax
            else:
                print("input skyRadmax is greater than image size")
            print("using Rmax = {:0.2f} ".format(Rend))

        if Rinit > Rend:

            Rinit= Rend/3 #is this number ok?
            print("Rinit is greater than image size")
            print("using Rinit = {:0.2f} ".format(Rinit))
            print("change this value with --radinit ")



        #mean, std, median  = self.GetRandBoxSky( Rinit, Rend )
        mean, var, median, N  = self.GetRandBoxSky2( Rinit, Rend )


        meansky = np.mean(mean)
        medsky = np.median(median)
        Npix = N.sum()
        stdsky = np.sqrt(var.sum()/Npix )


        #rmstd = stats.sem(mean)

        return meansky, stdsky,  medsky


    def GetXYRBorder(self):
        "this subroutine get the coordinates of the border"

        q =  self.q

        theta = self.thetadeg * (np.pi / 180)  # rads!!

        thetax=np.sqrt((np.cos(theta))**2 + (q**2)*(np.sin(theta))**2 )
        thetay=np.sqrt((q**2)*(np.cos(theta))**2 + (np.sin(theta))**2 )


        if (self.thetadeg >-45 and self.thetadeg <= 45):

            xmax=self.ncol
            xmin =1
            R1 = (xmax - self.xx)/thetax
            R2 = (self.xx - xmin)/thetax

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

        elif (self.thetadeg >45 and self.thetadeg <= 135):
            ymax=self.nrow
            ymin =1
            R1 = (ymax - self.yy)/thetay
            R2 = (self.yy - ymin)/thetay

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

        elif (self.thetadeg >135 and self.thetadeg <= 225):
            xmax=1
            xmin =self.ncol

            R1 = (xmax - self.xx)/thetax
            R2 = (self.xx - xmin)/thetax

            if (np.abs(R1)>=np.abs(R2)):
                R = R1
            else:
                R = R2

     
        elif (self.thetadeg >225 and self.thetadeg <= 315):
            ymax=1
            ymin =self.nrow
            R1 = (ymax - self.yy)/thetay
            R2 = (self.yy - ymin)/thetay

            if (np.abs(R1)>=np.abs(R2)):
               R = R1
            else:
               R = R2

        R = np.abs(R) #avoids negative numbers
        bim = q * R

        # getting size

        xmin = self.xx - np.sqrt((R**2) * (np.cos(theta))**2 +
                         (bim**2) * (np.sin(theta))**2)

        xmax = self.xx + np.sqrt((R**2) * (np.cos(theta))**2 +
                         (bim**2) * (np.sin(theta))**2)

        ymin = self.yy - np.sqrt((R**2) * (np.sin(theta))**2 +
                         (bim**2) * (np.cos(theta))**2)

        ymax = self.yy + np.sqrt((R**2) * (np.sin(theta))**2 +
                         (bim**2) * (np.cos(theta))**2)

        mask = xmin < 1
        if mask.any():
            if isinstance(xmin,np.ndarray):
                xmin[mask] = 1
            else:
                xmin = 1

        mask = xmax > self.ncol
        if mask.any():
            if isinstance(xmax,np.ndarray):
                xmax[mask] = self.ncol
            else:
                xmax = self.ncol

        mask = ymin < 1
        if mask.any():
            if isinstance(ymin,np.ndarray):
                ymin[mask] = 1
            else:
                ymin = 1

        mask = ymax > self.nrow
        if mask.any():
            if isinstance(ymax,np.ndarray):
                ymax[mask] = self.nrow
            else:
                ymax =self.nrow

        xmin=np.int32(np.round(xmin))
        ymin=np.int32(np.round(ymin))
        xmax=np.int32(np.round(xmax))
        ymax=np.int32(np.round(ymax))


        return (xmin,xmax,ymin,ymax,np.int32(R))

    def GetRandBoxSky(self, Rinit, Rmax):
        '''compute mean, std, and median from random selected boxes'''

        (nrow,ncol)=self.img.shape

        # it obtains corners of Rinit
        (xmino, xmaxo, ymino, ymaxo) = self.GetSize(self.xx, self.yy, Rinit, self.thetadeg, self.q, self.ncol, self.nrow) 

        # it obtains corners of Rmax
        (xminf, xmaxf, yminf, ymaxf) = self.GetSize(self.xx, self.yy, Rmax, self.thetadeg, self.q, self.ncol, self.nrow) 
        
        Value=1 #  value of counts  of the  main target for  the mask image  
        self.maskimg = self.MakeKron(self.maskimg, Value, self.xx, self.yy, Rinit, self.thetadeg, self.q, xminf, xmaxf, yminf, ymaxf) 

        ########

        sky = np.array([])
        skystd = np.array([])
        skymed = np.array([])



        coordinates = self.MakeCoord(xmino-self.box,xmaxo+self.box,ymino-self.box,ymaxo+self.box,xmaxf,ymaxf)


        cont=self.num # this is the number of boxes to use 

        for idx,item in enumerate(range(cont)):

            flatimg,xinit,yinit=self.GetRandomPatch(self.img,self.maskimg,self.box,coordinates)
            xfin = xinit + self.box - 1
            yfin = yinit + self.box - 1 


            flatimg.sort()

            boxcont=0

            while( not(flatimg.any()) and (boxcont < 10)):

                if (boxcont == 0):
                    print("Picking another box ")

                flatimg,xinit,yinit=self.GetRandomPatch(self.img,self.maskimg,self.box,coordinates)
                xfin = xinit + self.box - 1
                yfin = yinit + self.box - 1


                flatimg.sort()

                boxcont+=1 # avoid eternal loop

            if (boxcont == 10):
                print("max. iteration reached. I couldn't find a box")
     
            tot=len(flatimg)

            top=round(.8*tot)
            bot=round(.2*tot)
            
            #imgpatch=flatimg[bot:top]
            imgpatch=flatimg#[bot:top]

            linebox = "Box:{}   xinit/fin: {}-{}; yinit/fin: {}-{}  ".format(item+1,xinit,xfin,yinit,yfin)

            mean=np.mean(imgpatch)
            std=np.std(imgpatch)

            median=np.median(imgpatch)

            linemean = "sky  mean = {:.2f}; std = {:.2f}; median = {:.2f}".format(mean,std,median)
            print(linemean)

            sky=np.append(sky,mean)
            skystd=np.append(skystd,std)
            skymed=np.append(skymed,median)

        return sky,skystd,skymed


    def GetRandBoxSky2(self, Rinit, Rmax):
        '''compute mean, std, and median from random selected boxes'''

        (nrow,ncol)=self.img.shape

        # it obtains corners of Rinit
        (xmino, xmaxo, ymino, ymaxo) = self.GetSize(self.xx, self.yy, Rinit, self.thetadeg, self.q, self.ncol, self.nrow) 

        # it obtains corners of Rmax
        (xminf, xmaxf, yminf, ymaxf) = self.GetSize(self.xx, self.yy, Rmax, self.thetadeg, self.q, self.ncol, self.nrow) 
        
        Value=1 #  value of counts  of the  main target for  the mask image  
        self.maskimg = self.MakeKron(self.maskimg, Value, self.xx, self.yy, Rinit, self.thetadeg, self.q, xminf, xmaxf, yminf, ymaxf) 

        ########

        sky = np.array([])
        skystd = np.array([])
        skymed = np.array([])

        boximg = [] #list not numpy array

        N= np.array([])


        coordinates = self.MakeCoord(xmino-self.box,xmaxo+self.box,ymino-self.box,ymaxo+self.box,xmaxf,ymaxf)


        cont=self.num # this is the number of boxes to use 



        for idx,item in enumerate(range(cont)):

            flatimg,xinit,yinit=self.GetRandomPatch(self.img,self.maskimg,self.box,coordinates)
            xfin = xinit + self.box - 1
            yfin = yinit + self.box - 1 


            flatimg.sort()

            boxcont=0

            while( not(flatimg.any()) and (boxcont < 10)):

                if (boxcont == 0):
                    print("Picking another box ")

                flatimg,xinit,yinit=self.GetRandomPatch(self.img,self.maskimg,self.box,coordinates)
                xfin = xinit + self.box - 1
                yfin = yinit + self.box - 1


                flatimg.sort()

                boxcont+=1 # avoid eternal loop

            if (boxcont == 10):
                print("max. iteration reached. I couldn't find a box") 
                return 0,0,0 # It couldn't found any box; terminating
    

            linebox = "Box:{}   xinit/fin: {}-{}; yinit/fin: {}-{}  ".format(item+1,xinit,xfin,yinit,yfin)

            tot=len(flatimg)

            top=round(.8*tot)
            bot=round(.2*tot)
            
            if self.outliers:   # eliminate top 80% and bottom 20%
                imgpatch=flatimg[bot:top]
            else:
                imgpatch=flatimg

            boximg.append(imgpatch) #save for later
            N=np.append(N,imgpatch.size) #save for later

            mean=np.mean(imgpatch)
            std=np.std(imgpatch)

            median=np.median(imgpatch)

            linemean = "sky  mean = {:.2f}; std = {:.2f}; median = {:.2f}".format(mean,std,median)
            print(linemean)

            sky=np.append(sky,mean)
            #skystd=np.append(skystd,std)
            skymed=np.append(skymed,median)



        totmean = np.mean(sky)
        # to compute standard deviation:
        for idx, item in enumerate(range(cont)):

            bimg=boximg[idx]

            # not quite the var of the box, but it is needed to compute the TOTAL std:
            skyvar =  ((bimg-totmean)**2).sum() 

            skystd=np.append(skystd,skyvar)


        return sky,skystd,skymed,N

    def MakeCoord(self,xmino,xmaxo,ymino,ymaxo,xmaxf,ymaxf):
        ''' creates (x,y) coordinates between the inner and outer box'''

        if xmino < 1:
            xmino = 1

        if ymino < 1:
            ymino = 1

        if xmaxo > xmaxf:
            xmaxo = xmaxf

        if ymaxo > ymaxf:
            ymaxo = ymaxf




        coordinates = [(x,y) for x in np.arange(0,xmaxf) for y in np.arange(0,ymaxf) if not((x >= xmino and x <= xmaxo) and ( y >= ymino and y <= ymaxo))]

        return coordinates


    def MakeKron(self,imagemat, idn, x, y, R, theta, q, xmin, xmax, ymin, ymax):
        "This subroutine create a Kron ellipse within a box defined by: xmin, xmax, ymin, ymax"

        xmin = np.int(xmin)
        xmax = np.int(xmax)
        ymin = np.int(ymin)
        ymax = np.int(ymax)

        bim = q * R

        theta = theta * np.pi / 180  # Rads!!!

        ypos, xpos = np.mgrid[ymin - 1:ymax, xmin - 1:xmax]

        dx = xpos - x
        dy = ypos - y

        landa = np.arctan2(dy, dx)

        mask = landa < 0
        if mask.any():
            landa[mask] = landa[mask] + 2 * np.pi

        landa = landa - theta

        angle = np.arctan2(np.sin(landa) / bim, np.cos(landa) / R)

        xell = x + R * np.cos(angle) * np.cos(theta) - bim * \
            np.sin(angle) * np.sin(theta)
        yell = y + R * np.cos(angle) * np.sin(theta) + bim * \
            np.sin(angle) * np.cos(theta)

        dell = np.sqrt((xell - x)**2 + (yell - y)**2)
        dist = np.sqrt(dx**2 + dy**2)

        mask = dist < dell
        imagemat[ypos[mask], xpos[mask]] = idn

        return imagemat

    def GetSize(self,x, y, R, theta, q, ncol, nrow):
        '''this subroutine get the maximun
        and minimim pixels for Kron and sky ellipse'''

        bim = q * R

        theta = theta * (np.pi / 180)  # rads!!

        # getting size
        xmin = x - np.sqrt((R**2) * (np.cos(theta))**2 +
                           (bim**2) * (np.sin(theta))**2)

        xmax = x + np.sqrt((R**2) * (np.cos(theta))**2 +
                           (bim**2) * (np.sin(theta))**2)

        ymin = y - np.sqrt((R**2) * (np.sin(theta))**2 +
                           (bim**2) * (np.cos(theta))**2)

        ymax = y + np.sqrt((R**2) * (np.sin(theta))**2 +
                           (bim**2) * (np.cos(theta))**2)

        mask = xmin < 1
        if mask.any():
            if isinstance(xmin,np.ndarray):
                xmin[mask] = 1
            else:
                xmin = 1

        mask = xmax > ncol

        if mask.any():
            if isinstance(xmax,np.ndarray):
                xmax[mask] = ncol
            else:
                xmax = ncol

        mask = ymin < 1
        if mask.any():
            if isinstance(ymin,np.ndarray):
                ymin[mask] = 1
            else:
                ymin = 1

        mask = ymax > nrow
        if mask.any():
            if isinstance(ymax,np.ndarray):
                ymax[mask] = nrow
            else:
                ymax = nrow

        xmin=np.int32(np.round(xmin))
        ymin=np.int32(np.round(ymin))
        xmax=np.int32(np.round(xmax))
        ymax=np.int32(np.round(ymax))


        return (xmin, xmax, ymin, ymax)

    def GetRandomPatch(self,imagemat,mimg,box,coordinates):
        '''get a random box patch of the imagemat'''



        xinit,yinit=self.GetRandomCoord(coordinates)

        xfin = xinit + box  - 1 
        yfin = yinit + box  - 1

        imagebox=imagemat[yinit - 1:yfin, xinit - 1:xfin]
        maskbox=mimg[yinit - 1:yfin, xinit - 1:xfin]

        invboxpatch=np.logical_not(maskbox)

        return imagebox[invboxpatch],xinit,yinit



    def GetRandomCoord(self,coordinates):
        '''choose xinit, and yinit from coordinates'''
        
        ridx = np.random.randint(0,len(coordinates)-1)


        xinit = coordinates[ridx][0] 
        yinit = coordinates[ridx][1] 

        return xinit,yinit 

        ######


    def GetEllipSky(self, ImageFile, MaskFile, xx, yy, thetadeg, q, Rinit, width,namering,ringmask,outliers=True):
        "Gradient sky method"

        self.xx = xx 
        self.yy = yy

        self.thetadeg = 90 + thetadeg
        self.q = q
        self.e = (1 - self.q)
        self.Rinit = Rinit
        self.width = width 

        self.NumRings = 5  # number of rings per loop # read this from function?

        self.ringmask = ringmask 

        self.outliers = outliers

        ###

        #hdumask = fits.open(MaskFile)
        #self.maskimg = hdumask[0].data
        #hdumask.close()
        self.maskimg = MaskFile.copy()



        #hdu = fits.open(ImageFile)
        #self.img = hdu[0].data
        #hdu.close()
        self.img = ImageFile.copy()

        ####
       
        (self.nrow,self.ncol) = self.img.shape


        xmin,xmax,ymin,ymax,Rkron = self.GetXYRBorder()

        self.R = Rkron

        if self.Rinit > Rkron: # avoid radius greater than the border

            self.Rinit= Rkron/3 #is this number ok?
            print("Rinit is greater than image size")
            print("using Rinit = {:0.2f} ".format(self.Rinit))
            print("change this value with --radinit ")





        (xmin, xmax, ymin, ymax) = self.GetSize(self.xx, self.yy, Rkron, self.thetadeg, self.q, self.ncol, self.nrow) # obtain corners of R

        theta = self.thetadeg * np.pi / 180  # Rads!!!

        patch = self.maskimg[ymin - 1:ymax, xmin - 1:xmax] # logical patch mask image

        self.invpatch=np.logical_not(patch)


        Rings=np.arange(self.Rinit,self.R,self.width) # anillos de tamaño width

        sky = np.array([])
        skymed = np.array([])
        skystd = np.array([])
        radius = np.array([])


        count = 0
        idx=0


        ########################################
        #creating ring masks file

        masksky = np.zeros(np.shape(self.img))

        val = 1 
        for ridx, ritem in enumerate(Rings):


            bring = (Rings[ridx] + self.width) * self.q
            aring = Rings[ridx] + self.width 

            #incresing number of points per rad 
            points = 2*np.pi * np.sqrt(0.5*(aring**2 + bring**2)) #Aprox. 
            points = 2*points #doubling the number of points
            points = int(round(points))
           
            alpha = np.linspace(0,2*np.pi,points)
            for tidx, item in enumerate(range(self.width)):

                bim=(Rings[ridx]+item)*self.q

                tempxell = self.xx + (Rings[ridx]+item) * np.cos(alpha) * np.cos(theta) - bim * \
                  np.sin(alpha) * np.sin(theta)

                tempyell = self.yy + (Rings[ridx]+item) * np.cos(alpha) * np.sin(theta) + bim * \
                  np.sin(alpha) * np.cos(theta)

                tempxell = tempxell.round().astype("int")
                tempyell = tempyell.round().astype("int")

                tempxell,tempyell = self.CorSize(tempxell,tempyell)
                masksky[tempyell - 1, tempxell - 1] = val
                
            val += 1 


        #hdu[0].data=masksky
        #hdu.writeto(self.ringmask,overwrite=True) 

        ########################################
        ########################################



        #computing sky in every ring
        for ind, item in enumerate(Rings):


            maskring,idx=self.GetRingMask(masksky[ymin - 1:ymax, xmin - 1:xmax],idx)


            flatimg=self.img[ymin - 1:ymax, xmin - 1:xmax][maskring].flatten()  
            flatimg.sort()

            tot=len(flatimg)

            top=round(.8*tot)
            bot=round(.2*tot)


            if self.outliers:   # eliminate top 80% and bottom 20%
                imgpatch=flatimg[bot:top]
            else:
                imgpatch=flatimg

            mean=np.mean(imgpatch)
            std=np.std(imgpatch)

            median=np.median(imgpatch)

            sky=np.append(sky,mean)
            skymed=np.append(skymed,median)
            skystd=np.append(skystd,std)
            radius=np.append(radius,Rings[idx] + self.width/2)

            print("Ring = {}; rad = {:.2f}; sky mean = {:.2f}; sky std = {:.2f}; median: {:.2f} ".format(idx+1,Rings[idx] + self.width/2,mean,std, median))


            # calcular gradiente
            if (count >= self.NumRings):

                # [1:-1] avoiding the first and last element for gradient 
                gradmask = np.gradient(sky[1:-1]) >= 0 
               
                count = 0
                tempidx=np.where(np.gradient(sky[1:-1]) >= 0) 
                
                if (sky[1:-1][gradmask].any()): 
                    
                    savidx=tempidx[0][0]
                    maskring,none =self.GetRingMask(masksky[ymin - 1:ymax, xmin - 1:xmax],savidx)


                    print("sky computed in ring {} ".format(savidx+2))
                    
                    print("Ring radius = {:.2f} marked in {} ".format(radius[1:-1][savidx],namering))
                    print("the counts value within ring represent the long axis") 
                    self.img[ymin - 1:ymax, xmin - 1:xmax][maskring] = radius[1:-1][savidx] 
                    break

            count += 1
            idx +=1

            if idx == (len(Rings)-1): 
                print("The edge of image has been reached. Sky can not be computed")
                return 0,0,0,0


        #hdu[0].data=self.img
        #hdu.writeto(namering,overwrite=True) 

        finmean,finmedian,finstd,finRad = sky[1:-1][gradmask],skymed[1:-1][gradmask],skystd[1:-1][gradmask],radius[1:-1][gradmask]


        return finmean[0],finstd[0],finmedian[0],finRad[0]


    def GetRingMask(self,masksky,idx):
        ''' obtains the ring selected by index idx'''

        ring= idx + 1

        maskring = masksky == ring 

        maskring=maskring*self.invpatch

        ringcont=0

        while( not(maskring.any()) and (ringcont < 10)):

            if (ringcont == 0):
                    print("Selecting next ring ")

            idx += 1
            ring= idx + 1

            maskring = masksky == ring 

            maskring=maskring*self.invpatch


            ringcont+=1 # avoid eternal loop

        if (ringcont == 10):
            print("max. iteration reached. I couldn't find a ring") 
            return 0,0 # It couldn't found any ring ending 
 




        return maskring,idx


    def CorSize(self,xell,yell):
        '''Correct size for image borders'''
        masksx = xell < 1
        masksy = yell < 1

        xell[masksx] = 1
        yell[masksy] = 1

        masksx = xell > self.ncol
        masksy = yell > self.nrow 

        xell[masksx] = self.ncol 
        yell[masksy] = self.nrow

        return xell,yell

    ##### End of sky class ##################
    #########################################
    #########################################





#! /usr/bin/env python

import sys
import os.path




def readcon(params,confile):

    if not os.path.exists(confile):
        print ('%s: filename does not exist!' %sys.argv[1])
        sys.exit()


    with open(confile) as f_in:

        lines = (line.rstrip() for line in f_in) # All lines including the blank ones
        lines = (line.split('#', 1)[0] for line in lines) #remove comments
        lines = (line for line in lines if line) # Non-blank lines


        for line2 in lines:

            (param,val)=line2.split()

            # param 1
            flag = False

            if param == "FirstRun":

                (run1) = val.split()[0]
                params.run1=int(run1)

                flag = True 

            if param == "DEBLEND_NTHRESH1":
                (dn1) = val.split()[0]
                params.dn1=int(dn1)
                flag = True 

            if param == "DEBLEND_MINCONT1":
                (dm1) = val.split()[0]
                params.dm1=float(dm1)
                flag = True 


            if param == "ANALYSIS_THRESH1":

                (at1) = val.split()[0]
                params.at1=float(at1)
                flag = True 


            if param == "DETECT_THRESH1":

                (dt1) = val.split()[0]
                params.dt1= float(dt1)
                flag = True 

            if param == "DETECT_MINAREA1":

                (da1) = val.split()[0]
                params.da1=int(da1)
                flag = True 

            if param == "BACK_SIZE1":

                (bs1) = val.split()[0]
                params.bs1= float(bs1)
                flag = True 

            if param == "BACK_FILTERSIZE1":

                (bf1) = val.split()[0]
                params.bf1=int(bf1)
                flag = True 

             #  param2

            if param == "SecondRun":

                (run2) = val.split()[0]
                params.run2=int(run2)
                flag = True 


            if param == "DEBLEND_NTHRESH2":

                (dn2) = val.split()[0]
                params.dn2=float(dn2)
                flag = True 


            if param == "DEBLEND_MINCONT2":

                (dm2) = val.split()[0]
                params.dm2=float(dm2)
                flag = True 


            if param == "ANALYSIS_THRESH2":

                (at2) = val.split()[0]
                params.at2=float(at2)
                flag = True 


            if param == "DETECT_THRESH2":

                (dt2) = val.split()[0]
                params.dt2=float(dt2)
                flag = True 

            if param == "DETECT_MINAREA2":

                (da2) = val.split()[0]
                params.da2=int(da2)
                flag = True 

            if param == "BACK_SIZE2":

                (bs2) = val.split()[0]
                params.bs2= float(bs2)
                flag = True 

            if param == "BACK_FILTERSIZE2":

                (bf2) = val.split()[0]
                params.bf2=int(bf2)
                flag = True 



    # other options

            if param == "SatDs9":
                (satfileout) = val.split()[0]
                params.satfileout=str(satfileout)
                flag = True 

            if param == "SatScale":
                (satscale) = val.split()[0]
                params.satscale=float(satscale)
                flag = True 

            if param == "SatOffset":

                (satoffset) = val.split()[0]
                params.satoffset=int(satoffset)
                flag = True 

            if param == "SATUR_LEVEL":

                (satlevel) = val.split()[0]
                params.satlevel=int(satlevel)
                flag = True 


            #if param == "MakeMask":

            #    (create) = val.split()[0]
            #    params.create=int(create)
            #    flag = True 

            if param == "Scale":

                (scale) = val.split()[0]
                params.scale=float(scale)
                flag = True 

            if param == "Offset":

                (offset) = val.split()[0]
                params.offset=int(offset)
                flag = True 



            if param == "Scale2":

                (scale2) = val.split()[0]
                params.scale2=float(scale2)
                flag = True 


            if param == "OutCatalog":

                (output2) = val.split()[0]
                params.output2=str(output2)
                flag = True 

            if param == "MinSatSize":   # min size for sat regions

                (minsatsize) = val.split()[0]
                params.minsatsize=int(minsatsize)
                flag = True 

            if param == "SatQ":

                (satq) = val.split()[0]
                params.satq=float(satq)
                flag = True 


            if param == "Image":

                (image) = val.split()[0]
                params.image = str(image)
                flag = True 


            if param == "FracTol":   # proportion tolerance

                (tol) = val.split()[0]
                params.tol=float(tol)
                flag = True 


            if param == "ReduCoef":  #reduction factor

                (red) = val.split()[0]
                params.red =float(red)
                flag = True 

            if param == "MinRad":  #reduction factor

                (minrad) = val.split()[0]
                params.minrad =float(minrad)
                params.flagminrad = True
                flag = True 



            if param == "SatMethod":   #   method for detecting very bright saturated stars 

                (satmethod) = val.split()[0]
                params.satmethod=int(satmethod)
                flag = True 


        # SExtractor configuration parameters

            if param == "MAG_ZEROPOINT":  # zpt for sextractor  

                (zpt) = val.split()[0]
                params.zpt=float(zpt)
                flag = True 


            if param == "GAIN":   

                (gain) = val.split()[0]
                params.gain=float(gain)
                flag = True 

            if param == "PIXEL_SCALE":   

                (plate) = val.split()[0]
                params.plate=float(plate)
                flag = True 


            if param == "SEEING_FWHM":   

                (seeing) = val.split()[0]
                params.seeing=float(seeing)
                flag = True 



            if param == "JoinScale":
                (joinscale) = val.split()[0]
                params.joinscale=float(joinscale)
                flag = True 


            if param == "ScaleCor":

                (scalecor) = val.split()[0]
                params.scalecor=float(scalecor)
                flag = True 



            if param == "DisplayDs9":

                (ds9) = val.split()[0]
                params.flagDs9=int(ds9)
                flag = True 

            if param == "RegDs9":

                (regds9) = val.split()[0]
                params.regoutfile=str(regds9)
                flag = True 


            
            if flag == False: #parameter not recognized 

                msg="{} parameter not recognized".format(param) 
                print(msg)                 








#! /usr/bin/env python

import numpy as np
import argparse
import os
from astropy.io import fits
import subprocess as sp

from clusex.lib.join import joinsexcat 
from clusex.lib.join import putFlagSat 


from clusex.lib.init import printWelcome

from clusex.lib.check import GetAxis
from clusex.lib.make import CatArSort
from clusex.lib.make import MakeImage 
from clusex.lib.make import MakeMask
from clusex.lib.make import MakeSatBox 
from clusex.lib.make import MakeStamps
from clusex.lib.make import MakeObjImg


from clusex.lib.ds9 import ds9kron

from clusex.lib.make import EraseObjectMask2
from clusex.lib.make import EraseObjectMask


from clusex.lib.sky import SkyCal 



#console scripts



def joincat():
    """ joins two sextractor catalogs"""

    printWelcome()

    parser = argparse.ArgumentParser(description="Joincat: quickly combines two Sextractor catalogs")

    # required arguments
    parser.add_argument("FirstCatalog",help="First sextractor catalog")
    parser.add_argument("SecondCatalog",help="Second sextractor catalog")


    parser.add_argument("-s","--joinscale", type=float, help="factor that multiplies the radius of the First catalog objects. Objects outside of this catalog will be added. Default = 1",default=1)

    parser.add_argument("-o","--output", type=str, help="output catalog ",default='out.cat')

    parser.add_argument("-sf","--SatFile", type=str, help="Saturation DS9 reg file")

    parser.add_argument("-i","--include",action="store_true", help="Include all the galaxies from the second catalog that were not in the first catalog ")


    parser.add_argument("-r","--reduction", type=float, help="Reduction factor for objetc radius for -i option only",default=0.1)

    parser.add_argument("-m","--minrad", type=float, help="minrad for object radius for -i option only ",default=5)



    args = parser.parse_args()

    firstsex = args.FirstCatalog
    secondsex = args.SecondCatalog
    joinscale = args.joinscale
    output = args.output
    satfile = args.SatFile
    incFlag = args.include
    minrad= args.minrad
    red = args.reduction


    ##
    line="joining {} with {} using a scale of {}".format(firstsex,secondsex,joinscale)
    print(line)


    joinsexcat(firstsex,secondsex,output,joinscale,incFlag,red=red,minrad=minrad)


    if satfile:

        print("recomputing saturation flags for catalog")
        putFlagSat(output,"temp.cat",satfile)

        os.rename("temp.cat",output)

    else:

        print("no saturation DS9 reg file")

    line="joincat finished. output file {} created".format(output)
    print(line)


def makemask():
    """makes a mask from sextractor catalog"""


    printWelcome()

    parser = argparse.ArgumentParser(description="MakeMask: Creates mask from sextractor catalog")

    # required arguments
    parser.add_argument("SexCatalog",help="sextractor catalog")
    parser.add_argument("Image",help="Fits image of the objects")

    #optional arguments
    parser.add_argument("-s","--scale", type=float, help="factor that multiplies the radius of the catalog objects. Default = 1",default=1)
    
    parser.add_argument("-off","--offset", type=float, help="factor that it is added to the scale times radius of the catalog objects. Default = 0",default=0)

    parser.add_argument("-o","--outmask", type=str, help="name of the output mask ",default='mask.fits')

    parser.add_argument("-sf","--SatFile", type=str, help="Saturation DS9 reg file",default='ds9sat.reg')

    #options without arguments

    parser.add_argument("-n","--nodisplay",action="store_true", help="doesn't run DS9")


    args = parser.parse_args()

    sexcatalog = args.SexCatalog
    scale = args.scale
    offset = args.offset
    output = args.outmask
    satfile = args.SatFile
    image = args.Image
    flagds9 = args.nodisplay


    sexarsort="sexarea.cat"
    regoutfile="mask.reg"
    

    print ("Creating mask....\n")

    (NCol, NRow) = GetAxis(image)

    #check if exits satfile

    if not(os.path.exists(satfile)):
        with open(satfile, 'x') as f:
            f.write('box(1,1,0,0,0)') #to avoid empty file
            f.close()

    Total = CatArSort(sexcatalog,scale,offset,sexarsort,NCol,NRow)
   
    ##### segmentation mask

    MakeImage(output, NCol, NRow)

    MakeMask(output, sexarsort, scale, offset, satfile)  # offset set to 0
    MakeSatBox(output, satfile, Total + 1, NCol, NRow)

    #calling ds9kron to create ds9 reg objects
    ds9kron(sexcatalog,regoutfile,scale,offset)


    if not(flagds9): 
        print ("Running ds9 ...\n")
        runcmd="ds9 -tile column -cmap grey -invert -log -zmax -regions shape box {} -regions {} -regions {} {} ".format(image,regoutfile,satfile,output)
        err = sp.run([runcmd],shell=True,stdout=sp.PIPE,stderr=sp.PIPE,universal_newlines=True)  

    print('done') 


def remellmask():
    """Removes an ellipse from mask image"""


    printWelcome()

    parser = argparse.ArgumentParser(description="remellmask: Removes an ellipse from mask image")

    # required arguments
    parser.add_argument("mask",help="Fits mask image of the objects")

    parser.add_argument("number", type=int, help="ellipse number to be removed (or changed check option -f)")
    
    #optional arguments
    parser.add_argument("-f","--fill", type=float, help="Value number to be filled within the ellipse mask. 0 = object removed. Default = 0",default=0)

    parser.add_argument("-o","--outmask", type=str, help="name of the new mask ",default='objmask.fits')


    #options without arguments



    args = parser.parse_args()

    mask = args.mask
    number = args.number
    fill = args.fill
    output = args.outmask


    sexarsort="sexarea.cat"
    regoutfile="mask.reg"
    

    line="removing ellipse object:{} from mask: {}".format(number,mask)

    print (line)

    EraseObjectMask(mask,output,number,fill)

    print('new mask image:',output) 

    print('done')


def makestamps():
    """Make image stamps for each object"""


    printWelcome()

    parser = argparse.ArgumentParser(description="Make image stamps for every object in the catalog")

    # required arguments
    parser.add_argument("image",help="Fits image of the objects")
    parser.add_argument("catalog",help="sextractor catalog of the objects")
    parser.add_argument("mask",help="Fits image mask (created with makemask) ")

    #optional arguments
    parser.add_argument("-sr","--stretch", type=float, help="stretch factor to enlarge the stamps",default=5)

    parser.add_argument("-so","--skyoff", type=float, help="sky offset to be added the value of the mean sky ",default=1)


    parser.add_argument("-dp","--dpi", type=int, help="dots per inch resolution for image stamps",default=100)

    parser.add_argument("-cm","--cmap", type=str, help="color map",default='gray_r')


    parser.add_argument("-s","--scale", type=float, help="factor that multiplies the radius of the catalog objects. Default = 1",default=1)
    
    parser.add_argument("-off","--offset", type=float, help="factor that it is added to the scale times radius of the catalog objects. Default = 0",default=0)


    parser.add_argument("-br","--bright", type=float, help="brightness of the image. Default = 33",default=33)

    parser.add_argument("-co","--contrast", type=float, help="contrast of the image. Default = 0.98",default=0.98)


    parser.add_argument("-gc","--galclass", type=float, help="galaxy/star sextractor classification limit. Sextractor Classification  1 = Star. 0 = galaxy",default=1)

    args = parser.parse_args()

    image= args.image
    catalog= args.catalog
    mask = args.mask
    stretch = args.stretch
    skyoff = args.skyoff
    dpi= args.dpi
    cmap = args.cmap

    scale = args.scale
    offset  = args.offset
    bright = args.bright
    contrast = args.contrast
    galclass= args.galclass



    line="Creating image stamps for every object of the catalog"

    print (line)

    MakeStamps(image, catalog, mask, stretch, skyoff, dpi, cmap, 
                scale, offset, bright, contrast, galclass)


    print('done')




def sex2ds9():
    """creates a ds9 reg file from sextractor catalog"""

    printWelcome()


    parser = argparse.ArgumentParser(description="sex2ds9: Creates a DS9 reg file from sextractor catalog")

    # required arguments
    parser.add_argument("SexCatalog",help="sextractor catalog")

    #optional arguments
    parser.add_argument("-s","--scale", type=float, help="factor that multiplies the radius of the catalog objects. Default = 1",default=1)
    
    parser.add_argument("-off","--offset", type=float, help="factor that it is added to the scale times radius of the catalog objects. Default = 0",default=0)


    parser.add_argument("-o","--outreg", type=str, help="name of the output DS9 reg file ",default='ds9.reg')


    args = parser.parse_args()

    sexcatalog = args.SexCatalog
    scale = args.scale
    offset = args.offset
    regoutfile = args.outreg
 
    print("Creating DS9 reg file: ",regoutfile)

    ds9kron(sexcatalog,regoutfile,scale,offset)

    print("done.")
    

def makeobjimg():
    """Make objects image """

    printWelcome()

    parser = argparse.ArgumentParser(description="Make object image")

    # required arguments
    parser.add_argument("image",help="Fits image of the objects")
    parser.add_argument("mask",help="Fits image mask (created with makemask) ")

    parser.add_argument("-o","--outimage", type=str, help="name of the output obj image file ",default='objects.fits')

    args = parser.parse_args()

    image= args.image
    maskimage = args.mask
    outimage = args.outimage
 


    hdu = fits.open(image)
    img = hdu[0].data

    hdumask = fits.open(maskimage)
    mask = hdumask[0].data
    hdumask.close()


    objimage = MakeObjImg(img, mask)


    hdu[0].data = objimage 
    hdu.writeto(outimage, overwrite=True)
    hdu.close()


    print('new objects image:',outimage) 

    print("done")


def compsky():
    """Computes the sky for every object of a sextractor catalog"""

    printWelcome()

    parser = argparse.ArgumentParser(description="skysex: computes the sky for every object of a sextractor catalog")

    # required arguments
    parser.add_argument("SexCatalog",help="sextractor catalog")
    parser.add_argument("Image",help="Fits image of the objects")

    parser.add_argument("MaskFile",help="Fits mask image. File created with makemask")


    parser.add_argument("-s","--scaleRadius", type=float, help="factor that multiplies the radius of the catalog objects. For grad sky it is the Inital radius. For rand sky is the minimum radius of the box around main object. Default = 1",default=1)
   
    parser.add_argument("-w","--width", type=int, help="width of the ring for the grad method. ",default=20)

    parser.add_argument("-b","--box", type=int, help="size of the box for the random method. ",default=20)


    parser.add_argument("-nb","--numBox", type=int, help="number of boxes for the random method. ",default=20)

    parser.add_argument("-sm","--scaleRadMax", type=float, help="factor that multiplies the radius of the catalog objects. For rand sky is the maximum radius of the box around main object. Default = 20",default=20)
 

    parser.add_argument("-m","--method", type=int, help="method used for compute sky. Grad sky =1, rand sky = 2. Default = 1  ",default=1)


    parser.add_argument("-o","--outcat", type=str, help="name of the output catalog ",default='out.cat')




    args = parser.parse_args()

    sexcatalog = args.SexCatalog
    image = args.Image
    mask = args.MaskFile
    scale = args.scaleRadius
    width = args.width

    box = args.box
    numbox = args.numBox

    scaleMax = args.scaleRadMax

    method = args.method

    output = args.outcat


    ##end input




    ### reading sex catalog:
    N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(sexcatalog,delimiter="",unpack=True)

    AR         = 1 - E
    RKron      = scale * Ai * Kr
    RKronMax      = scaleMax * Ai * Kr

    Theta = Theta + 90 #it must be the same as GALFIT 

    maskron = RKron <= 0
    RKron[maskron] = 1

    maskar = AR <= 0.005

    AR[maskar] = 0.005


    

    #######################################
    ############## SKY ####################
    #######################################

    f_out = open(output, "w")

    #  gradient sky method:
    if method == 1:


        # computing sky with the gradient method
        print("Computing sky with the gradient method")

        ImageFile = image
        MaskFile = mask

        tempMask = "tempmask.fits"
        #width = params.skywidth

        hdumask = fits.open(MaskFile)
        maskimg = hdumask[0].data
        hdumask.close()

        hdu = fits.open(ImageFile)
        datimg = hdu[0].data
        hdu.close()


       
        for idx, item in enumerate(N):

            xx = X[idx]
            yy = Y[idx] 


            thetadeg = Theta[idx]
            q = AR[idx]
            Rinit = RKron[idx]


            print("computing sky for object ",N[idx])

            line="using Rinit = {:.2f} width = {}".format(Rinit,width)
            print(line)

            line="using thetadeg = {:.2f} q = {:.2f}".format(thetadeg,q)
            print(line)
        
            line="using x = {} y  = {}".format(xx,yy)
            print(line)

            #EraseObjectMask(MaskFile,tempMask,N[idx])
            tempmask=EraseObjectMask2(maskimg,N[idx]) #routine more efficient 


            #mean,std, median,rad = SkyCal().GetEllipSky(ImageFile,tempMask,xx,yy,
            #                                            thetadeg,q,Rinit,width,
            #                                            "ring.fits","ringmask.fits")
            mean,std, median,rad = SkyCal().GetEllipSky(datimg,tempmask,xx,yy,
                                                        thetadeg,q,Rinit,width,
                                                        "ring.fits","ringmask.fits")


            line="Total sky:  mean = {:.2f}; std={:.2f}; median = {:.2f} ".format(mean,std,median)
            print(line)

            #saving for output
            Bkgd[idx] = mean  
            #galpar.gradskystd = std
            #galpar.gradskymed = median


            line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11:.2f} {12:.2f} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])


            f_out.write(line)



    #  random sky method:
    elif method == 2:

        # computing sky  using random boxes across the image
        print("computing sky with the random box method")

        ImageFile = image
        MaskFile = mask

        hdu = fits.open(ImageFile)
        datimg = hdu[0].data
        hdu.close()


        hdumask = fits.open(MaskFile)
        maskimg = hdumask[0].data
        hdumask.close()





        for idx, item in enumerate(N):

            xx = X[idx]
            yy = Y[idx] 


            thetadeg = Theta[idx]
            q = AR[idx]
            Rinit = RKron[idx]
            Rmax = RKronMax[idx]


            ###

            #box = params.skybox
            #num = params.skynum
            #box = args.box
            #numbox = args.numBox

            #scaleMax = args.scaleRadMax

            print("computing sky for object ",N[idx])
            line="using Rad = {:.2f}, box size= {}, number of boxes = {}".format(Rinit,box,numbox)
            print(line)


           # mean,std, median = SkyCal().RandBox(ImageFile,MaskFile,xx,yy,
           #                                     thetadeg,q,Rinit,box,numbox,Rmax)
            mean,std, median = SkyCal().RandBox(datimg,maskimg,xx,yy,
                                                thetadeg,q,Rinit,box,numbox,Rmax)

            line="Total sky:  mean = {:.2f}; std = {:.2f}; median = {:.2f}".format(mean,std,median)
            print(line)

            #saving for output
            Bkgd[idx] = mean  
            #galpar.randskystd = std
            #galpar.randskymed = median
            line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11:.2f} {12:.2f} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])


            f_out.write(line)



    else: 

        # computing sky  using random boxes across the image
        print("method not found. use '-m 1' for grad sky and '-m 2' for random sky")




    f_out.close()


    print("computation of sky has finished")

    #######################################
    ############## SKY End ################
    #######################################






#! /usr/bin/env python

import numpy as np
from astropy.io import fits
from astropy.wcs import WCS
#from astropy.utils.data import get_pkg_data_filename

def CheckFlag(val,check):
   "Check for flag contained in $val, returns 1 if found "

   flag = False
   mod = 1
   max=128


   while (mod != 0):

       res = int(val/max)

       if (max == check and res == 1 ):

           flag=True

       mod = val % max

       val = mod
       max = max/2


   return flag


def CheckSatReg2(x,y,filein):
   "Check if object is inside of saturated region. returns True if at least one pixel is inside"
## check if object is inside of
## saturaded region as indicated by ds9 box region
## returns True if object center is in saturaded region

   flag = False

   with open(filein) as f_in:

       lines = (line.rstrip() for line in f_in) # All lines including the blank ones
       lines = (line.split('#', 1)[0] for line in lines) # remove comments
       lines = (line.rstrip() for line in lines)   # remove lines containing only comments
       lines = (line for line in lines if line) # Non-blank lines

       for line in lines:

           if (line != "image"):

               (box,info)=line.split('(')

               if(box == "box"):

                   (xpos,ypos,xlong,ylong,trash)=info.split(',')

                   xpos=float(xpos)
                   ypos=float(ypos)
                   xlong=float(xlong)
                   ylong=float(ylong)


                   xlo = xpos - xlong/2
                   xhi = xpos + xlong/2

                   ylo = ypos - ylong/2
                   yhi = ypos + ylong/2

                   if ( (x > xlo and x < xhi) and (y > ylo and y < yhi) ):
                       flag=True
                       break


   return flag


def CheckKron (xpos,ypos,x,y,R,theta,q):
    "check if position is inside of the Kron Ellipse saturaded region returns True if object center is in Ellipse region"


    bim = q * R

    theta = theta * np.pi /180  ## Rads!!!


    flag =False


    dx = xpos - x
    dy = ypos - y


    landa = np.arctan2( dy,dx )

    if landa < 0:
        landa=landa + 2 * np.pi


    landa = landa - theta


    angle = np.arctan2(np.sin(landa)/bim, np.cos(landa)/R)

    xell =  x + R * np.cos(angle)* np.cos(theta)  - bim * np.sin(angle) * np.sin(theta)
    yell =  y + R * np.cos(angle)* np.sin(theta)  + bim * np.sin(angle) * np.cos(theta)

    dell = np.sqrt ( (xell - x)**2 + (yell - y)**2 )
    dist = np.sqrt ( dx**2 + dy**2 )


    if  dist < dell:
	      flag=True


    return flag


def GetAxis(Image):
    # k Check
    "Get number of rows and columns from the image"

    hdu = fits.open(Image)
    ncol = hdu[0].header["NAXIS1"]
    nrow = hdu[0].header["NAXIS2"]
    hdu.close()
    return ncol, nrow

def GetWCS(Image):
    # k Check
    "Get World Cordinate System info"
    hdu = fits.open(Image)[0]
    wcs = WCS(hdu.header)

    return wcs 

def GetCounts(Image):
    # k Check
    "Get Counts from Image"

    hdu = fits.open(Image)[0]
    counts = hdu.data.ravel()

    return counts







#! /usr/bin/env python

import numpy as np

from clusex.lib.check import CheckFlag 


def ds9kron(sexfile,regfile,scale,offset):
    "Creates ds9 region file to check output catalog "



    f_out= open(regfile, "w")

    #    scale = 1
    #offset=0


    flagsat=4      ## flag value when object is saturated (or close to)


    #print OUT "image \n";


    line="image \n"
    f_out.write(line)

    count=0

    N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Star,Flg=np.genfromtxt(sexfile,delimiter="",unpack=True)

    for idx, item in enumerate(N):

        Rkron = scale * Ai[idx] * Kr[idx] + offset

       

        if Rkron == 0:

            Rkron = 1

        q = (1 - E)
        bim = q * Rkron


        check=CheckFlag(Flg[idx],flagsat)  ## check if object doesn't has saturated regions


        if  (check == False) :


            line="ellipse({0},{1},{2},{3},{4}) # color=blue move=0 \n".format(X[idx],Y[idx],Rkron,bim[idx],Theta[idx])

            f_out.write(line)


            line2="point({0},{1}) # point=boxcircle font=\"times 10 bold\" text={2} {3} {4} \n".format(X[idx],Y[idx],"{",np.int(N[idx]),"}")

            f_out.write(line2)

        else:
            count +=1

            line="ellipse({0},{1},{2},{3},{4}) # color=red move=0 \n".format(X[idx],Y[idx],Rkron,bim[idx],Theta[idx])

            #f_out.write(line)


            line2="point({0},{1}) # point=boxcircle color=red font=\"times 10 bold\" text={2} {3} {4} \n".format(X[idx],Y[idx],"{",np.int(N[idx]),"}")

            f_out.write(line2)




    print ("{} objects have at least one saturated pixels  \n".format(count))


    #        f_out.write(line)



    f_out.close()







#! /usr/bin/env python

import numpy as np




class RadMod:

    def __init__(self, sexcat1: str, sexcat2: str, newcat: str, tol=1, red=1, 
                minrad=10,scalecor=1):
        """
        This routine modifies wrong large estimated radius for Sextractor catalogs. 
        It does so by comparing the radius of two catalogs of the same image. It 
        kept the smallest radius of the two catalogs.
        The objects radius in sexcat1 will be changed for the ones if sexcat2 if the
        tolerance is achieved. newcat will be the modified catalog
        """

        #first cat
        N,Alpha,Delta,X,Y,Mg,Kr,Fluxr,Isoa,Ai,E,Theta,Bkgd,Idx,Flg=np.genfromtxt(sexcat1,delimiter="",unpack=True)


        #second cat
        N2,Alpha2,Delta2,X2,Y2,Mg2,Kr2,Fluxr2,Isoa2,Ai2,E2,Theta2,Bkgd2,Idx2,Flg2=np.genfromtxt(sexcat2,delimiter="",unpack=True)


        dmax= 3 # max distance to match the same object in different catalogs 

        count = 0
        count2 = 0

        for idx, item in enumerate(N):

            foundflag = False

            dx = X2 - X[idx]
            dy = Y2 - Y[idx]
            dist = np.sqrt(dx**2 + dy**2)

            distmin = dist.min()
            idx2 = dist.argmin()

            if distmin <= dmax:

                foundflag = True
                #cflag1 = self.CheckFlag(Flg[idx],4) #check for saturated flag 
                #cflag2 = self.CheckFlag(Flg2[idx2],4) #check for saturated flag 

                cflag1 = False 
                cflag2 = False 


                #the comparison is based in Fluxr parameter

                #rad = Fluxr[idx]
                #rad2 = Fluxr2[idx2]

                # or  you can use kr * ai

                rad = Kr[idx] * Ai[idx]
                rad2 =  Kr2[idx2] *  Ai2[idx2]

                kr1 = Kr[idx]
                kr2 = Kr2[idx2]

                ai1 = Ai[idx]
                ai2 = Ai2[idx2]


                if rad2 < .1: 
                    rad2 = .1 #avoids division by zero


                den = rad2 

                num = rad - rad2

                comp = num/den

                if ((cflag1  and cflag2) == False): 

                    if comp > tol:

                        count +=1 

                        # reduction factor included
                        #Kr[idx],Fluxr[idx],Isoa[idx],Ai[idx] = red * Kr2[idx2],Fluxr2[idx2],Isoa2[idx2],Ai2[idx2]
                        Kr[idx],Fluxr[idx],Isoa[idx],Ai[idx] =  Kr2[idx2],Fluxr2[idx2],Isoa2[idx2],Ai2[idx2]

                        #Ai[idx] =  Ai2[idx2]
                        #Kr[idx] =  Kr2[idx2]


                        #this is to avoid galaxies with very low radius after interchanging
                        #radius. However we don't want to increase it to a larger 
                        #radius than the previous one:

                        #kr2 is the smallest radii of the two 
                        

                        comp = rad/(scalecor*rad2)  - 1
                        if comp > tol:
                            Kr[idx] = scalecor*Kr[idx]
                            #Ai[idx] = scalecor*Ai[idx]
                        

            if foundflag == False:

                # If object was not found in the other catalog, then
                # the Kr is reduced by a factor "red". Final radius
                # can not be less than minrad
                # This is done to avoid faint large galaxies.
 

                Kr[idx]= red * Kr[idx]

                if Kr[idx]*Ai[idx] < minrad:    

                    Kr[idx] = minrad/Ai[idx]
        
                count2 +=1 

        

        line = "catalog {}: {} objects with modified radius ".format(sexcat1,count)
        print(line)


        line = "catalog {}: {} objects with reduced radius ".format(sexcat1,count2)
        print(line)


        #writing catalogs

        fout = open(newcat, "w")

        for idx, item in enumerate(N):

            line="{0:.0f} {1} {2} {3} {4} {5} {6} {7} {8:.0f} {9} {10} {11} {12} {13} {14:.0f} \n".format(N[idx], Alpha[idx], Delta[idx], X[idx], Y[idx], Mg[idx], Kr[idx], Fluxr[idx], Isoa[idx], Ai[idx], E[idx], Theta[idx], Bkgd[idx], Idx[idx], Flg[idx])

            fout.write(line)

        fout.close()



    def CheckFlag(self,val: int, check: int) -> bool:
        "Check for flag contained in val, returns True if found "

        flag = False
        mod = 1
        maxx=128


        while (mod != 0):

            res = int(val / maxx)

            if (maxx == check and res == 1):

                flag = True

            mod = val % maxx

            val = mod
            maxx = maxx / 2

        return flag


